/*
 *  Copyright (c) 2000-2022 Inria
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  * Neither the name of the ALICE Project-Team nor the names of its
 *  contributors may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *  Contact: Bruno Levy
 *
 *     https://www.inria.fr/fr/bruno-levy
 *
 *     Inria,
 *     Domaine de Voluceau,
 *     78150 Le Chesnay - Rocquencourt
 *     FRANCE
 *
 */

#include <geogram_gfx/imgui_ext/icon_font.h>
#include <geogram/basic/string.h>
#include <map>
#include <string>

namespace {

/*
  Table below generated from metadata in FontAwesome distrib:
  cat icons.yml | awk 'BEGIN { state = 0; } {
  if(state == 0 && $1 != "voted:") {
  key = $1;
  gsub(":","",key);
  state = 1;
  } else if(state == 1 && $1 == "unicode:") {
  printf("font_awesome_table[\"%s\"] = 0x%s;\n",key,$2)
  state = 0;
  }
  }'
*/

    // Aliases:
    // "edit"          -> "pen-to-square"
    // "mouse-pointer" -> "arrow-pointer"
    // "window-close"  -> "rectangle-xmark"
    // "cog"           -> "gear"
    // "sync-alt"      -> "rotate"
    // "save"          -> "floppy-disk"
    // "home"          -> "house"

    std::map<std::string,wchar_t> font_awesome_table;
    void init_font_awesome_table(void);

    void init_font_awesome_table() {
        static bool initialized = false;
        if(initialized) {
            return;
        }
        initialized = true;
	font_awesome_table["'0'"] = 0x30;
	font_awesome_table["'1'"] = 0x31;
	font_awesome_table["'2'"] = 0x32;
	font_awesome_table["'3'"] = 0x33;
	font_awesome_table["'4'"] = 0x34;
	font_awesome_table["'5'"] = 0x35;
	font_awesome_table["'6'"] = 0x36;
	font_awesome_table["'7'"] = 0x37;
	font_awesome_table["'8'"] = 0x38;
	font_awesome_table["'9'"] = 0x39;
	font_awesome_table["42-group"] = 0xe080;
	font_awesome_table["500px"] = 0xf26e;
	font_awesome_table["a"] = 0x41;
	font_awesome_table["accessible-icon"] = 0xf368;
	font_awesome_table["accusoft"] = 0xf369;
	font_awesome_table["address-book"] = 0xf2b9;
	font_awesome_table["address-card"] = 0xf2bb;
	font_awesome_table["adn"] = 0xf170;
	font_awesome_table["adversal"] = 0xf36a;
	font_awesome_table["affiliatetheme"] = 0xf36b;
	font_awesome_table["airbnb"] = 0xf834;
	font_awesome_table["alarm-clock"] = 0xf34e;
	font_awesome_table["algolia"] = 0xf36c;
	font_awesome_table["align-center"] = 0xf037;
	font_awesome_table["align-justify"] = 0xf039;
	font_awesome_table["align-left"] = 0xf036;
	font_awesome_table["align-right"] = 0xf038;
	font_awesome_table["alipay"] = 0xf642;
	font_awesome_table["amazon"] = 0xf270;
	font_awesome_table["amazon-pay"] = 0xf42c;
	font_awesome_table["amilia"] = 0xf36d;
	font_awesome_table["anchor"] = 0xf13d;
	font_awesome_table["anchor-circle-check"] = 0xe4aa;
	font_awesome_table["anchor-circle-exclamation"] = 0xe4ab;
	font_awesome_table["anchor-circle-xmark"] = 0xe4ac;
	font_awesome_table["anchor-lock"] = 0xe4ad;
	font_awesome_table["android"] = 0xf17b;
	font_awesome_table["angellist"] = 0xf209;
	font_awesome_table["angle-down"] = 0xf107;
	font_awesome_table["angle-left"] = 0xf104;
	font_awesome_table["angle-right"] = 0xf105;
	font_awesome_table["angle-up"] = 0xf106;
	font_awesome_table["angles-down"] = 0xf103;
	font_awesome_table["angles-left"] = 0xf100;
	font_awesome_table["angles-right"] = 0xf101;
	font_awesome_table["angles-up"] = 0xf102;
	font_awesome_table["angrycreative"] = 0xf36e;
	font_awesome_table["angular"] = 0xf420;
	font_awesome_table["ankh"] = 0xf644;
	font_awesome_table["app-store"] = 0xf36f;
	font_awesome_table["app-store-ios"] = 0xf370;
	font_awesome_table["apper"] = 0xf371;
	font_awesome_table["apple"] = 0xf179;
	font_awesome_table["apple-pay"] = 0xf415;
	font_awesome_table["apple-whole"] = 0xf5d1;
	font_awesome_table["aquarius"] = 0xe845;
	font_awesome_table["arch-linux"] = 0xe867;
	font_awesome_table["archway"] = 0xf557;
	font_awesome_table["aries"] = 0xe846;
	font_awesome_table["arrow-down"] = 0xf063;
	font_awesome_table["arrow-down-1-9"] = 0xf162;
	font_awesome_table["arrow-down-9-1"] = 0xf886;
	font_awesome_table["arrow-down-a-z"] = 0xf15d;
	font_awesome_table["arrow-down-long"] = 0xf175;
	font_awesome_table["arrow-down-short-wide"] = 0xf884;
	font_awesome_table["arrow-down-up-across-line"] = 0xe4af;
	font_awesome_table["arrow-down-up-lock"] = 0xe4b0;
	font_awesome_table["arrow-down-wide-short"] = 0xf160;
	font_awesome_table["arrow-down-z-a"] = 0xf881;
	font_awesome_table["arrow-left"] = 0xf060;
	font_awesome_table["arrow-left-long"] = 0xf177;
	font_awesome_table["arrow-pointer"] = 0xf245;
	font_awesome_table["mouse-pointer"] = 0xf245;
	font_awesome_table["arrow-right"] = 0xf061;
	font_awesome_table["arrow-right-arrow-left"] = 0xf0ec;
	font_awesome_table["arrow-right-from-bracket"] = 0xf08b;
	font_awesome_table["arrow-right-long"] = 0xf178;
	font_awesome_table["arrow-right-to-bracket"] = 0xf090;
	font_awesome_table["arrow-right-to-city"] = 0xe4b3;
	font_awesome_table["arrow-rotate-left"] = 0xf0e2;
	font_awesome_table["arrow-rotate-right"] = 0xf01e;
	font_awesome_table["arrow-trend-down"] = 0xe097;
	font_awesome_table["arrow-trend-up"] = 0xe098;
	font_awesome_table["arrow-turn-down"] = 0xf149;
	font_awesome_table["arrow-turn-up"] = 0xf148;
	font_awesome_table["arrow-up"] = 0xf062;
	font_awesome_table["arrow-up-1-9"] = 0xf163;
	font_awesome_table["arrow-up-9-1"] = 0xf887;
	font_awesome_table["arrow-up-a-z"] = 0xf15e;
	font_awesome_table["arrow-up-from-bracket"] = 0xe09a;
	font_awesome_table["arrow-up-from-ground-water"] = 0xe4b5;
	font_awesome_table["arrow-up-from-water-pump"] = 0xe4b6;
	font_awesome_table["arrow-up-long"] = 0xf176;
	font_awesome_table["arrow-up-right-dots"] = 0xe4b7;
	font_awesome_table["arrow-up-right-from-square"] = 0xf08e;
	font_awesome_table["arrow-up-short-wide"] = 0xf885;
	font_awesome_table["arrow-up-wide-short"] = 0xf161;
	font_awesome_table["arrow-up-z-a"] = 0xf882;
	font_awesome_table["arrows-down-to-line"] = 0xe4b8;
	font_awesome_table["arrows-down-to-people"] = 0xe4b9;
	font_awesome_table["arrows-left-right"] = 0xf07e;
	font_awesome_table["arrows-left-right-to-line"] = 0xe4ba;
	font_awesome_table["arrows-rotate"] = 0xf021;
	font_awesome_table["arrows-spin"] = 0xe4bb;
	font_awesome_table["arrows-split-up-and-left"] = 0xe4bc;
	font_awesome_table["arrows-to-circle"] = 0xe4bd;
	font_awesome_table["arrows-to-dot"] = 0xe4be;
	font_awesome_table["arrows-to-eye"] = 0xe4bf;
	font_awesome_table["arrows-turn-right"] = 0xe4c0;
	font_awesome_table["arrows-turn-to-dots"] = 0xe4c1;
	font_awesome_table["arrows-up-down"] = 0xf07d;
	font_awesome_table["arrows-up-down-left-right"] = 0xf047;
	font_awesome_table["arrows-up-to-line"] = 0xe4c2;
	font_awesome_table["artstation"] = 0xf77a;
	font_awesome_table["asterisk"] = 0x2a;
	font_awesome_table["asymmetrik"] = 0xf372;
	font_awesome_table["at"] = 0x40;
	font_awesome_table["atlassian"] = 0xf77b;
	font_awesome_table["atom"] = 0xf5d2;
	font_awesome_table["audible"] = 0xf373;
	font_awesome_table["audio-description"] = 0xf29e;
	font_awesome_table["austral-sign"] = 0xe0a9;
	font_awesome_table["autoprefixer"] = 0xf41c;
	font_awesome_table["avianex"] = 0xf374;
	font_awesome_table["aviato"] = 0xf421;
	font_awesome_table["award"] = 0xf559;
	font_awesome_table["aws"] = 0xf375;
	font_awesome_table["b"] = 0x42;
	font_awesome_table["baby"] = 0xf77c;
	font_awesome_table["baby-carriage"] = 0xf77d;
	font_awesome_table["backward"] = 0xf04a;
	font_awesome_table["backward-fast"] = 0xf049;
	font_awesome_table["backward-step"] = 0xf048;
	font_awesome_table["bacon"] = 0xf7e5;
	font_awesome_table["bacteria"] = 0xe059;
	font_awesome_table["bacterium"] = 0xe05a;
	font_awesome_table["bag-shopping"] = 0xf290;
	font_awesome_table["bahai"] = 0xf666;
	font_awesome_table["baht-sign"] = 0xe0ac;
	font_awesome_table["ban"] = 0xf05e;
	font_awesome_table["ban-smoking"] = 0xf54d;
	font_awesome_table["bandage"] = 0xf462;
	font_awesome_table["bandcamp"] = 0xf2d5;
	font_awesome_table["bangladeshi-taka-sign"] = 0xe2e6;
	font_awesome_table["barcode"] = 0xf02a;
	font_awesome_table["bars"] = 0xf0c9;
	font_awesome_table["bars-progress"] = 0xf828;
	font_awesome_table["bars-staggered"] = 0xf550;
	font_awesome_table["baseball"] = 0xf433;
	font_awesome_table["baseball-bat-ball"] = 0xf432;
	font_awesome_table["basket-shopping"] = 0xf291;
	font_awesome_table["basketball"] = 0xf434;
	font_awesome_table["bath"] = 0xf2cd;
	font_awesome_table["battery-empty"] = 0xf244;
	font_awesome_table["battery-full"] = 0xf240;
	font_awesome_table["battery-half"] = 0xf242;
	font_awesome_table["battery-quarter"] = 0xf243;
	font_awesome_table["battery-three-quarters"] = 0xf241;
	font_awesome_table["battle-net"] = 0xf835;
	font_awesome_table["bed"] = 0xf236;
	font_awesome_table["bed-pulse"] = 0xf487;
	font_awesome_table["beer-mug-empty"] = 0xf0fc;
	font_awesome_table["behance"] = 0xf1b4;
	font_awesome_table["bell"] = 0xf0f3;
	font_awesome_table["bell-concierge"] = 0xf562;
	font_awesome_table["bell-slash"] = 0xf1f6;
	font_awesome_table["bezier-curve"] = 0xf55b;
	font_awesome_table["bicycle"] = 0xf206;
	font_awesome_table["bilibili"] = 0xe3d9;
	font_awesome_table["bimobject"] = 0xf378;
	font_awesome_table["binoculars"] = 0xf1e5;
	font_awesome_table["biohazard"] = 0xf780;
	font_awesome_table["bitbucket"] = 0xf171;
	font_awesome_table["bitcoin"] = 0xf379;
	font_awesome_table["bitcoin-sign"] = 0xe0b4;
	font_awesome_table["bity"] = 0xf37a;
	font_awesome_table["black-tie"] = 0xf27e;
	font_awesome_table["blackberry"] = 0xf37b;
	font_awesome_table["blender"] = 0xf517;
	font_awesome_table["blender-phone"] = 0xf6b6;
	font_awesome_table["blog"] = 0xf781;
	font_awesome_table["blogger"] = 0xf37c;
	font_awesome_table["blogger-b"] = 0xf37d;
	font_awesome_table["bluesky"] = 0xe671;
	font_awesome_table["bluetooth"] = 0xf293;
	font_awesome_table["bluetooth-b"] = 0xf294;
	font_awesome_table["board-game-geek"] = 0xe855;
	font_awesome_table["bold"] = 0xf032;
	font_awesome_table["bolt"] = 0xf0e7;
	font_awesome_table["bolt-lightning"] = 0xe0b7;
	font_awesome_table["bomb"] = 0xf1e2;
	font_awesome_table["bone"] = 0xf5d7;
	font_awesome_table["bong"] = 0xf55c;
	font_awesome_table["book"] = 0xf02d;
	font_awesome_table["book-atlas"] = 0xf558;
	font_awesome_table["book-bible"] = 0xf647;
	font_awesome_table["book-bookmark"] = 0xe0bb;
	font_awesome_table["book-journal-whills"] = 0xf66a;
	font_awesome_table["book-medical"] = 0xf7e6;
	font_awesome_table["book-open"] = 0xf518;
	font_awesome_table["book-open-reader"] = 0xf5da;
	font_awesome_table["book-quran"] = 0xf687;
	font_awesome_table["book-skull"] = 0xf6b7;
	font_awesome_table["book-tanakh"] = 0xf827;
	font_awesome_table["bookmark"] = 0xf02e;
	font_awesome_table["bootstrap"] = 0xf836;
	font_awesome_table["border-all"] = 0xf84c;
	font_awesome_table["border-none"] = 0xf850;
	font_awesome_table["border-top-left"] = 0xf853;
	font_awesome_table["bore-hole"] = 0xe4c3;
	font_awesome_table["bots"] = 0xe340;
	font_awesome_table["bottle-droplet"] = 0xe4c4;
	font_awesome_table["bottle-water"] = 0xe4c5;
	font_awesome_table["bowl-food"] = 0xe4c6;
	font_awesome_table["bowl-rice"] = 0xe2eb;
	font_awesome_table["bowling-ball"] = 0xf436;
	font_awesome_table["box"] = 0xf466;
	font_awesome_table["box-archive"] = 0xf187;
	font_awesome_table["box-open"] = 0xf49e;
	font_awesome_table["box-tissue"] = 0xe05b;
	font_awesome_table["boxes-packing"] = 0xe4c7;
	font_awesome_table["boxes-stacked"] = 0xf468;
	font_awesome_table["braille"] = 0xf2a1;
	font_awesome_table["brain"] = 0xf5dc;
	font_awesome_table["brave"] = 0xe63c;
	font_awesome_table["brave-reverse"] = 0xe63d;
	font_awesome_table["brazilian-real-sign"] = 0xe46c;
	font_awesome_table["bread-slice"] = 0xf7ec;
	font_awesome_table["bridge"] = 0xe4c8;
	font_awesome_table["bridge-circle-check"] = 0xe4c9;
	font_awesome_table["bridge-circle-exclamation"] = 0xe4ca;
	font_awesome_table["bridge-circle-xmark"] = 0xe4cb;
	font_awesome_table["bridge-lock"] = 0xe4cc;
	font_awesome_table["bridge-water"] = 0xe4ce;
	font_awesome_table["briefcase"] = 0xf0b1;
	font_awesome_table["briefcase-medical"] = 0xf469;
	font_awesome_table["broom"] = 0xf51a;
	font_awesome_table["broom-ball"] = 0xf458;
	font_awesome_table["brush"] = 0xf55d;
	font_awesome_table["btc"] = 0xf15a;
	font_awesome_table["bucket"] = 0xe4cf;
	font_awesome_table["buffer"] = 0xf837;
	font_awesome_table["bug"] = 0xf188;
	font_awesome_table["bug-slash"] = 0xe490;
	font_awesome_table["bugs"] = 0xe4d0;
	font_awesome_table["building"] = 0xf1ad;
	font_awesome_table["building-circle-arrow-right"] = 0xe4d1;
	font_awesome_table["building-circle-check"] = 0xe4d2;
	font_awesome_table["building-circle-exclamation"] = 0xe4d3;
	font_awesome_table["building-circle-xmark"] = 0xe4d4;
	font_awesome_table["building-columns"] = 0xf19c;
	font_awesome_table["building-flag"] = 0xe4d5;
	font_awesome_table["building-lock"] = 0xe4d6;
	font_awesome_table["building-ngo"] = 0xe4d7;
	font_awesome_table["building-shield"] = 0xe4d8;
	font_awesome_table["building-un"] = 0xe4d9;
	font_awesome_table["building-user"] = 0xe4da;
	font_awesome_table["building-wheat"] = 0xe4db;
	font_awesome_table["bullhorn"] = 0xf0a1;
	font_awesome_table["bullseye"] = 0xf140;
	font_awesome_table["burger"] = 0xf805;
	font_awesome_table["buromobelexperte"] = 0xf37f;
	font_awesome_table["burst"] = 0xe4dc;
	font_awesome_table["bus"] = 0xf207;
	font_awesome_table["bus-side"] = 0xe81d;
	font_awesome_table["bus-simple"] = 0xf55e;
	font_awesome_table["business-time"] = 0xf64a;
	font_awesome_table["buy-n-large"] = 0xf8a6;
	font_awesome_table["buysellads"] = 0xf20d;
	font_awesome_table["c"] = 0x43;
	font_awesome_table["cable-car"] = 0xf7da;
	font_awesome_table["cake-candles"] = 0xf1fd;
	font_awesome_table["calculator"] = 0xf1ec;
	font_awesome_table["calendar"] = 0xf133;
	font_awesome_table["calendar-check"] = 0xf274;
	font_awesome_table["calendar-day"] = 0xf783;
	font_awesome_table["calendar-days"] = 0xf073;
	font_awesome_table["calendar-minus"] = 0xf272;
	font_awesome_table["calendar-plus"] = 0xf271;
	font_awesome_table["calendar-week"] = 0xf784;
	font_awesome_table["calendar-xmark"] = 0xf273;
	font_awesome_table["camera"] = 0xf030;
	font_awesome_table["camera-retro"] = 0xf083;
	font_awesome_table["camera-rotate"] = 0xe0d8;
	font_awesome_table["campground"] = 0xf6bb;
	font_awesome_table["canadian-maple-leaf"] = 0xf785;
	font_awesome_table["cancer"] = 0xe847;
	font_awesome_table["candy-cane"] = 0xf786;
	font_awesome_table["cannabis"] = 0xf55f;
	font_awesome_table["capricorn"] = 0xe848;
	font_awesome_table["capsules"] = 0xf46b;
	font_awesome_table["car"] = 0xf1b9;
	font_awesome_table["car-battery"] = 0xf5df;
	font_awesome_table["car-burst"] = 0xf5e1;
	font_awesome_table["car-on"] = 0xe4dd;
	font_awesome_table["car-rear"] = 0xf5de;
	font_awesome_table["car-side"] = 0xf5e4;
	font_awesome_table["car-tunnel"] = 0xe4de;
	font_awesome_table["caravan"] = 0xf8ff;
	font_awesome_table["caret-down"] = 0xf0d7;
	font_awesome_table["caret-left"] = 0xf0d9;
	font_awesome_table["caret-right"] = 0xf0da;
	font_awesome_table["caret-up"] = 0xf0d8;
	font_awesome_table["carrot"] = 0xf787;
	font_awesome_table["cart-arrow-down"] = 0xf218;
	font_awesome_table["cart-flatbed"] = 0xf474;
	font_awesome_table["cart-flatbed-suitcase"] = 0xf59d;
	font_awesome_table["cart-plus"] = 0xf217;
	font_awesome_table["cart-shopping"] = 0xf07a;
	font_awesome_table["cash-app"] = 0xe7d4;
	font_awesome_table["cash-register"] = 0xf788;
	font_awesome_table["cat"] = 0xf6be;
	font_awesome_table["cc-amazon-pay"] = 0xf42d;
	font_awesome_table["cc-amex"] = 0xf1f3;
	font_awesome_table["cc-apple-pay"] = 0xf416;
	font_awesome_table["cc-diners-club"] = 0xf24c;
	font_awesome_table["cc-discover"] = 0xf1f2;
	font_awesome_table["cc-jcb"] = 0xf24b;
	font_awesome_table["cc-mastercard"] = 0xf1f1;
	font_awesome_table["cc-paypal"] = 0xf1f4;
	font_awesome_table["cc-stripe"] = 0xf1f5;
	font_awesome_table["cc-visa"] = 0xf1f0;
	font_awesome_table["cedi-sign"] = 0xe0df;
	font_awesome_table["cent-sign"] = 0xe3f5;
	font_awesome_table["centercode"] = 0xf380;
	font_awesome_table["centos"] = 0xf789;
	font_awesome_table["certificate"] = 0xf0a3;
	font_awesome_table["chair"] = 0xf6c0;
	font_awesome_table["chalkboard"] = 0xf51b;
	font_awesome_table["chalkboard-user"] = 0xf51c;
	font_awesome_table["champagne-glasses"] = 0xf79f;
	font_awesome_table["charging-station"] = 0xf5e7;
	font_awesome_table["chart-area"] = 0xf1fe;
	font_awesome_table["chart-bar"] = 0xf080;
	font_awesome_table["chart-column"] = 0xe0e3;
	font_awesome_table["chart-diagram"] = 0xe695;
	font_awesome_table["chart-gantt"] = 0xe0e4;
	font_awesome_table["chart-line"] = 0xf201;
	font_awesome_table["chart-pie"] = 0xf200;
	font_awesome_table["chart-simple"] = 0xe473;
	font_awesome_table["check"] = 0xf00c;
	font_awesome_table["check-double"] = 0xf560;
	font_awesome_table["check-to-slot"] = 0xf772;
	font_awesome_table["cheese"] = 0xf7ef;
	font_awesome_table["chess"] = 0xf439;
	font_awesome_table["chess-bishop"] = 0xf43a;
	font_awesome_table["chess-board"] = 0xf43c;
	font_awesome_table["chess-king"] = 0xf43f;
	font_awesome_table["chess-knight"] = 0xf441;
	font_awesome_table["chess-pawn"] = 0xf443;
	font_awesome_table["chess-queen"] = 0xf445;
	font_awesome_table["chess-rook"] = 0xf447;
	font_awesome_table["chevron-down"] = 0xf078;
	font_awesome_table["chevron-left"] = 0xf053;
	font_awesome_table["chevron-right"] = 0xf054;
	font_awesome_table["chevron-up"] = 0xf077;
	font_awesome_table["child"] = 0xf1ae;
	font_awesome_table["child-combatant"] = 0xe4e0;
	font_awesome_table["child-dress"] = 0xe59c;
	font_awesome_table["child-reaching"] = 0xe59d;
	font_awesome_table["children"] = 0xe4e1;
	font_awesome_table["chrome"] = 0xf268;
	font_awesome_table["chromecast"] = 0xf838;
	font_awesome_table["church"] = 0xf51d;
	font_awesome_table["circle"] = 0xf111;
	font_awesome_table["circle-arrow-down"] = 0xf0ab;
	font_awesome_table["circle-arrow-left"] = 0xf0a8;
	font_awesome_table["circle-arrow-right"] = 0xf0a9;
	font_awesome_table["circle-arrow-up"] = 0xf0aa;
	font_awesome_table["circle-check"] = 0xf058;
	font_awesome_table["circle-chevron-down"] = 0xf13a;
	font_awesome_table["circle-chevron-left"] = 0xf137;
	font_awesome_table["circle-chevron-right"] = 0xf138;
	font_awesome_table["circle-chevron-up"] = 0xf139;
	font_awesome_table["circle-dollar-to-slot"] = 0xf4b9;
	font_awesome_table["circle-dot"] = 0xf192;
	font_awesome_table["circle-down"] = 0xf358;
	font_awesome_table["circle-exclamation"] = 0xf06a;
	font_awesome_table["circle-h"] = 0xf47e;
	font_awesome_table["circle-half-stroke"] = 0xf042;
	font_awesome_table["circle-info"] = 0xf05a;
	font_awesome_table["circle-left"] = 0xf359;
	font_awesome_table["circle-minus"] = 0xf056;
	font_awesome_table["circle-nodes"] = 0xe4e2;
	font_awesome_table["circle-notch"] = 0xf1ce;
	font_awesome_table["circle-pause"] = 0xf28b;
	font_awesome_table["circle-play"] = 0xf144;
	font_awesome_table["circle-plus"] = 0xf055;
	font_awesome_table["circle-question"] = 0xf059;
	font_awesome_table["circle-radiation"] = 0xf7ba;
	font_awesome_table["circle-right"] = 0xf35a;
	font_awesome_table["circle-stop"] = 0xf28d;
	font_awesome_table["circle-up"] = 0xf35b;
	font_awesome_table["circle-user"] = 0xf2bd;
	font_awesome_table["circle-xmark"] = 0xf057;
	font_awesome_table["circle-zulip"] = 0xe851;
	font_awesome_table["city"] = 0xf64f;
	font_awesome_table["clapperboard"] = 0xe131;
	font_awesome_table["claude"] = 0xe861;
	font_awesome_table["clipboard"] = 0xf328;
	font_awesome_table["clipboard-check"] = 0xf46c;
	font_awesome_table["clipboard-list"] = 0xf46d;
	font_awesome_table["clipboard-question"] = 0xe4e3;
	font_awesome_table["clipboard-user"] = 0xf7f3;
	font_awesome_table["clock"] = 0xf017;
	font_awesome_table["clock-rotate-left"] = 0xf1da;
	font_awesome_table["clone"] = 0xf24d;
	font_awesome_table["closed-captioning"] = 0xf20a;
	font_awesome_table["closed-captioning-slash"] = 0xe135;
	font_awesome_table["cloud"] = 0xf0c2;
	font_awesome_table["cloud-arrow-down"] = 0xf0ed;
	font_awesome_table["cloud-arrow-up"] = 0xf0ee;
	font_awesome_table["cloud-bolt"] = 0xf76c;
	font_awesome_table["cloud-meatball"] = 0xf73b;
	font_awesome_table["cloud-moon"] = 0xf6c3;
	font_awesome_table["cloud-moon-rain"] = 0xf73c;
	font_awesome_table["cloud-rain"] = 0xf73d;
	font_awesome_table["cloud-showers-heavy"] = 0xf740;
	font_awesome_table["cloud-showers-water"] = 0xe4e4;
	font_awesome_table["cloud-sun"] = 0xf6c4;
	font_awesome_table["cloud-sun-rain"] = 0xf743;
	font_awesome_table["cloudflare"] = 0xe07d;
	font_awesome_table["cloudscale"] = 0xf383;
	font_awesome_table["cloudsmith"] = 0xf384;
	font_awesome_table["cloudversify"] = 0xf385;
	font_awesome_table["clover"] = 0xe139;
	font_awesome_table["cmplid"] = 0xe360;
	font_awesome_table["code"] = 0xf121;
	font_awesome_table["code-branch"] = 0xf126;
	font_awesome_table["code-commit"] = 0xf386;
	font_awesome_table["code-compare"] = 0xe13a;
	font_awesome_table["code-fork"] = 0xe13b;
	font_awesome_table["code-merge"] = 0xf387;
	font_awesome_table["code-pull-request"] = 0xe13c;
	font_awesome_table["codepen"] = 0xf1cb;
	font_awesome_table["codiepie"] = 0xf284;
	font_awesome_table["coins"] = 0xf51e;
	font_awesome_table["colon-sign"] = 0xe140;
	font_awesome_table["comment"] = 0xf075;
	font_awesome_table["comment-dollar"] = 0xf651;
	font_awesome_table["comment-dots"] = 0xf4ad;
	font_awesome_table["comment-medical"] = 0xf7f5;
	font_awesome_table["comment-nodes"] = 0xe696;
	font_awesome_table["comment-slash"] = 0xf4b3;
	font_awesome_table["comment-sms"] = 0xf7cd;
	font_awesome_table["comments"] = 0xf086;
	font_awesome_table["comments-dollar"] = 0xf653;
	font_awesome_table["compact-disc"] = 0xf51f;
	font_awesome_table["compass"] = 0xf14e;
	font_awesome_table["compass-drafting"] = 0xf568;
	font_awesome_table["compress"] = 0xf066;
	font_awesome_table["computer"] = 0xe4e5;
	font_awesome_table["computer-mouse"] = 0xf8cc;
	font_awesome_table["confluence"] = 0xf78d;
	font_awesome_table["connectdevelop"] = 0xf20e;
	font_awesome_table["contao"] = 0xf26d;
	font_awesome_table["cookie"] = 0xf563;
	font_awesome_table["cookie-bite"] = 0xf564;
	font_awesome_table["copy"] = 0xf0c5;
	font_awesome_table["copyright"] = 0xf1f9;
	font_awesome_table["cotton-bureau"] = 0xf89e;
	font_awesome_table["couch"] = 0xf4b8;
	font_awesome_table["cow"] = 0xf6c8;
	font_awesome_table["cpanel"] = 0xf388;
	font_awesome_table["creative-commons"] = 0xf25e;
	font_awesome_table["creative-commons-by"] = 0xf4e7;
	font_awesome_table["creative-commons-nc"] = 0xf4e8;
	font_awesome_table["creative-commons-nc-eu"] = 0xf4e9;
	font_awesome_table["creative-commons-nc-jp"] = 0xf4ea;
	font_awesome_table["creative-commons-nd"] = 0xf4eb;
	font_awesome_table["creative-commons-pd"] = 0xf4ec;
	font_awesome_table["creative-commons-pd-alt"] = 0xf4ed;
	font_awesome_table["creative-commons-remix"] = 0xf4ee;
	font_awesome_table["creative-commons-sa"] = 0xf4ef;
	font_awesome_table["creative-commons-sampling"] = 0xf4f0;
	font_awesome_table["creative-commons-sampling-plus"] = 0xf4f1;
	font_awesome_table["creative-commons-share"] = 0xf4f2;
	font_awesome_table["creative-commons-zero"] = 0xf4f3;
	font_awesome_table["credit-card"] = 0xf09d;
	font_awesome_table["critical-role"] = 0xf6c9;
	font_awesome_table["crop"] = 0xf125;
	font_awesome_table["crop-simple"] = 0xf565;
	font_awesome_table["cross"] = 0xf654;
	font_awesome_table["crosshairs"] = 0xf05b;
	font_awesome_table["crow"] = 0xf520;
	font_awesome_table["crown"] = 0xf521;
	font_awesome_table["crutch"] = 0xf7f7;
	font_awesome_table["cruzeiro-sign"] = 0xe152;
	font_awesome_table["css"] = 0xe6a2;
	font_awesome_table["css3"] = 0xf13c;
	font_awesome_table["css3-alt"] = 0xf38b;
	font_awesome_table["cube"] = 0xf1b2;
	font_awesome_table["cubes"] = 0xf1b3;
	font_awesome_table["cubes-stacked"] = 0xe4e6;
	font_awesome_table["cuttlefish"] = 0xf38c;
	font_awesome_table["d"] = 0x44;
	font_awesome_table["d-and-d"] = 0xf38d;
	font_awesome_table["d-and-d-beyond"] = 0xf6ca;
	font_awesome_table["dailymotion"] = 0xe052;
	font_awesome_table["dart-lang"] = 0xe693;
	font_awesome_table["dashcube"] = 0xf210;
	font_awesome_table["database"] = 0xf1c0;
	font_awesome_table["debian"] = 0xe60b;
	font_awesome_table["deezer"] = 0xe077;
	font_awesome_table["delete-left"] = 0xf55a;
	font_awesome_table["delicious"] = 0xf1a5;
	font_awesome_table["democrat"] = 0xf747;
	font_awesome_table["deploydog"] = 0xf38e;
	font_awesome_table["deskpro"] = 0xf38f;
	font_awesome_table["desktop"] = 0xf390;
	font_awesome_table["dev"] = 0xf6cc;
	font_awesome_table["deviantart"] = 0xf1bd;
	font_awesome_table["dharmachakra"] = 0xf655;
	font_awesome_table["dhl"] = 0xf790;
	font_awesome_table["diagram-next"] = 0xe476;
	font_awesome_table["diagram-predecessor"] = 0xe477;
	font_awesome_table["diagram-project"] = 0xf542;
	font_awesome_table["diagram-successor"] = 0xe47a;
	font_awesome_table["diamond"] = 0xf219;
	font_awesome_table["diamond-turn-right"] = 0xf5eb;
	font_awesome_table["diaspora"] = 0xf791;
	font_awesome_table["dice"] = 0xf522;
	font_awesome_table["dice-d20"] = 0xf6cf;
	font_awesome_table["dice-d6"] = 0xf6d1;
	font_awesome_table["dice-five"] = 0xf523;
	font_awesome_table["dice-four"] = 0xf524;
	font_awesome_table["dice-one"] = 0xf525;
	font_awesome_table["dice-six"] = 0xf526;
	font_awesome_table["dice-three"] = 0xf527;
	font_awesome_table["dice-two"] = 0xf528;
	font_awesome_table["digg"] = 0xf1a6;
	font_awesome_table["digital-ocean"] = 0xf391;
	font_awesome_table["discord"] = 0xf392;
	font_awesome_table["discourse"] = 0xf393;
	font_awesome_table["disease"] = 0xf7fa;
	font_awesome_table["display"] = 0xe163;
	font_awesome_table["disqus"] = 0xe7d5;
	font_awesome_table["divide"] = 0xf529;
	font_awesome_table["dna"] = 0xf471;
	font_awesome_table["dochub"] = 0xf394;
	font_awesome_table["docker"] = 0xf395;
	font_awesome_table["dog"] = 0xf6d3;
	font_awesome_table["dollar-sign"] = 0x24;
	font_awesome_table["dolly"] = 0xf472;
	font_awesome_table["dong-sign"] = 0xe169;
	font_awesome_table["door-closed"] = 0xf52a;
	font_awesome_table["door-open"] = 0xf52b;
	font_awesome_table["dove"] = 0xf4ba;
	font_awesome_table["down-left-and-up-right-to-center"] = 0xf422;
	font_awesome_table["down-long"] = 0xf309;
	font_awesome_table["download"] = 0xf019;
	font_awesome_table["draft2digital"] = 0xf396;
	font_awesome_table["dragon"] = 0xf6d5;
	font_awesome_table["draw-polygon"] = 0xf5ee;
	font_awesome_table["dribbble"] = 0xf17d;
	font_awesome_table["dropbox"] = 0xf16b;
	font_awesome_table["droplet"] = 0xf043;
	font_awesome_table["droplet-slash"] = 0xf5c7;
	font_awesome_table["drum"] = 0xf569;
	font_awesome_table["drum-steelpan"] = 0xf56a;
	font_awesome_table["drumstick-bite"] = 0xf6d7;
	font_awesome_table["drupal"] = 0xf1a9;
	font_awesome_table["dumbbell"] = 0xf44b;
	font_awesome_table["dumpster"] = 0xf793;
	font_awesome_table["dumpster-fire"] = 0xf794;
	font_awesome_table["dungeon"] = 0xf6d9;
	font_awesome_table["duolingo"] = 0xe812;
	font_awesome_table["dyalog"] = 0xf399;
	font_awesome_table["e"] = 0x45;
	font_awesome_table["ear-deaf"] = 0xf2a4;
	font_awesome_table["ear-listen"] = 0xf2a2;
	font_awesome_table["earlybirds"] = 0xf39a;
	font_awesome_table["earth-africa"] = 0xf57c;
	font_awesome_table["earth-americas"] = 0xf57d;
	font_awesome_table["earth-asia"] = 0xf57e;
	font_awesome_table["earth-europe"] = 0xf7a2;
	font_awesome_table["earth-oceania"] = 0xe47b;
	font_awesome_table["ebay"] = 0xf4f4;
	font_awesome_table["edge"] = 0xf282;
	font_awesome_table["edge-legacy"] = 0xe078;
	font_awesome_table["egg"] = 0xf7fb;
	font_awesome_table["eject"] = 0xf052;
	font_awesome_table["elementor"] = 0xf430;
	font_awesome_table["elevator"] = 0xe16d;
	font_awesome_table["eleventy"] = 0xe7d6;
	font_awesome_table["ellipsis"] = 0xf141;
	font_awesome_table["ellipsis-vertical"] = 0xf142;
	font_awesome_table["ello"] = 0xf5f1;
	font_awesome_table["ember"] = 0xf423;
	font_awesome_table["empire"] = 0xf1d1;
	font_awesome_table["envelope"] = 0xf0e0;
	font_awesome_table["envelope-circle-check"] = 0xe4e8;
	font_awesome_table["envelope-open"] = 0xf2b6;
	font_awesome_table["envelope-open-text"] = 0xf658;
	font_awesome_table["envelopes-bulk"] = 0xf674;
	font_awesome_table["envira"] = 0xf299;
	font_awesome_table["equals"] = 0x3d;
	font_awesome_table["eraser"] = 0xf12d;
	font_awesome_table["erlang"] = 0xf39d;
	font_awesome_table["ethereum"] = 0xf42e;
	font_awesome_table["ethernet"] = 0xf796;
	font_awesome_table["etsy"] = 0xf2d7;
	font_awesome_table["euro-sign"] = 0xf153;
	font_awesome_table["evernote"] = 0xf839;
	font_awesome_table["exclamation"] = 0x21;
	font_awesome_table["expand"] = 0xf065;
	font_awesome_table["expeditedssl"] = 0xf23e;
	font_awesome_table["explosion"] = 0xe4e9;
	font_awesome_table["eye"] = 0xf06e;
	font_awesome_table["eye-dropper"] = 0xf1fb;
	font_awesome_table["eye-low-vision"] = 0xf2a8;
	font_awesome_table["eye-slash"] = 0xf070;
	font_awesome_table["f"] = 0x46;
	font_awesome_table["face-angry"] = 0xf556;
	font_awesome_table["face-dizzy"] = 0xf567;
	font_awesome_table["face-flushed"] = 0xf579;
	font_awesome_table["face-frown"] = 0xf119;
	font_awesome_table["face-frown-open"] = 0xf57a;
	font_awesome_table["face-grimace"] = 0xf57f;
	font_awesome_table["face-grin"] = 0xf580;
	font_awesome_table["face-grin-beam"] = 0xf582;
	font_awesome_table["face-grin-beam-sweat"] = 0xf583;
	font_awesome_table["face-grin-hearts"] = 0xf584;
	font_awesome_table["face-grin-squint"] = 0xf585;
	font_awesome_table["face-grin-squint-tears"] = 0xf586;
	font_awesome_table["face-grin-stars"] = 0xf587;
	font_awesome_table["face-grin-tears"] = 0xf588;
	font_awesome_table["face-grin-tongue"] = 0xf589;
	font_awesome_table["face-grin-tongue-squint"] = 0xf58a;
	font_awesome_table["face-grin-tongue-wink"] = 0xf58b;
	font_awesome_table["face-grin-wide"] = 0xf581;
	font_awesome_table["face-grin-wink"] = 0xf58c;
	font_awesome_table["face-kiss"] = 0xf596;
	font_awesome_table["face-kiss-beam"] = 0xf597;
	font_awesome_table["face-kiss-wink-heart"] = 0xf598;
	font_awesome_table["face-laugh"] = 0xf599;
	font_awesome_table["face-laugh-beam"] = 0xf59a;
	font_awesome_table["face-laugh-squint"] = 0xf59b;
	font_awesome_table["face-laugh-wink"] = 0xf59c;
	font_awesome_table["face-meh"] = 0xf11a;
	font_awesome_table["face-meh-blank"] = 0xf5a4;
	font_awesome_table["face-rolling-eyes"] = 0xf5a5;
	font_awesome_table["face-sad-cry"] = 0xf5b3;
	font_awesome_table["face-sad-tear"] = 0xf5b4;
	font_awesome_table["face-smile"] = 0xf118;
	font_awesome_table["face-smile-beam"] = 0xf5b8;
	font_awesome_table["face-smile-wink"] = 0xf4da;
	font_awesome_table["face-surprise"] = 0xf5c2;
	font_awesome_table["face-tired"] = 0xf5c8;
	font_awesome_table["facebook"] = 0xf09a;
	font_awesome_table["facebook-f"] = 0xf39e;
	font_awesome_table["facebook-messenger"] = 0xf39f;
	font_awesome_table["fan"] = 0xf863;
	font_awesome_table["fantasy-flight-games"] = 0xf6dc;
	font_awesome_table["faucet"] = 0xe005;
	font_awesome_table["faucet-drip"] = 0xe006;
	font_awesome_table["fax"] = 0xf1ac;
	font_awesome_table["feather"] = 0xf52d;
	font_awesome_table["feather-pointed"] = 0xf56b;
	font_awesome_table["fedex"] = 0xf797;
	font_awesome_table["fediverse"] = 0xe865;
	font_awesome_table["fedora"] = 0xf798;
	font_awesome_table["ferry"] = 0xe4ea;
	font_awesome_table["figma"] = 0xf799;
	font_awesome_table["file"] = 0xf15b;
	font_awesome_table["file-arrow-down"] = 0xf56d;
	font_awesome_table["file-arrow-up"] = 0xf574;
	font_awesome_table["file-audio"] = 0xf1c7;
	font_awesome_table["file-circle-check"] = 0xe5a0;
	font_awesome_table["file-circle-exclamation"] = 0xe4eb;
	font_awesome_table["file-circle-minus"] = 0xe4ed;
	font_awesome_table["file-circle-plus"] = 0xe494;
	font_awesome_table["file-circle-question"] = 0xe4ef;
	font_awesome_table["file-circle-xmark"] = 0xe5a1;
	font_awesome_table["file-code"] = 0xf1c9;
	font_awesome_table["file-contract"] = 0xf56c;
	font_awesome_table["file-csv"] = 0xf6dd;
	font_awesome_table["file-excel"] = 0xf1c3;
	font_awesome_table["file-export"] = 0xf56e;
	font_awesome_table["file-fragment"] = 0xe697;
	font_awesome_table["file-half-dashed"] = 0xe698;
	font_awesome_table["file-image"] = 0xf1c5;
	font_awesome_table["file-import"] = 0xf56f;
	font_awesome_table["file-invoice"] = 0xf570;
	font_awesome_table["file-invoice-dollar"] = 0xf571;
	font_awesome_table["file-lines"] = 0xf15c;
	font_awesome_table["file-medical"] = 0xf477;
	font_awesome_table["file-pdf"] = 0xf1c1;
	font_awesome_table["file-pen"] = 0xf31c;
	font_awesome_table["file-powerpoint"] = 0xf1c4;
	font_awesome_table["file-prescription"] = 0xf572;
	font_awesome_table["file-shield"] = 0xe4f0;
	font_awesome_table["file-signature"] = 0xf573;
	font_awesome_table["file-video"] = 0xf1c8;
	font_awesome_table["file-waveform"] = 0xf478;
	font_awesome_table["file-word"] = 0xf1c2;
	font_awesome_table["file-zipper"] = 0xf1c6;
	font_awesome_table["files-pinwheel"] = 0xe69f;
	font_awesome_table["fill"] = 0xf575;
	font_awesome_table["fill-drip"] = 0xf576;
	font_awesome_table["film"] = 0xf008;
	font_awesome_table["filter"] = 0xf0b0;
	font_awesome_table["filter-circle-dollar"] = 0xf662;
	font_awesome_table["filter-circle-xmark"] = 0xe17b;
	font_awesome_table["fingerprint"] = 0xf577;
	font_awesome_table["fire"] = 0xf06d;
	font_awesome_table["fire-burner"] = 0xe4f1;
	font_awesome_table["fire-extinguisher"] = 0xf134;
	font_awesome_table["fire-flame-curved"] = 0xf7e4;
	font_awesome_table["fire-flame-simple"] = 0xf46a;
	font_awesome_table["firefox"] = 0xf269;
	font_awesome_table["firefox-browser"] = 0xe007;
	font_awesome_table["first-order"] = 0xf2b0;
	font_awesome_table["first-order-alt"] = 0xf50a;
	font_awesome_table["firstdraft"] = 0xf3a1;
	font_awesome_table["fish"] = 0xf578;
	font_awesome_table["fish-fins"] = 0xe4f2;
	font_awesome_table["flag"] = 0xf024;
	font_awesome_table["flag-checkered"] = 0xf11e;
	font_awesome_table["flag-usa"] = 0xf74d;
	font_awesome_table["flask"] = 0xf0c3;
	font_awesome_table["flask-vial"] = 0xe4f3;
	font_awesome_table["flickr"] = 0xf16e;
	font_awesome_table["flipboard"] = 0xf44d;
	font_awesome_table["floppy-disk"] = 0xf0c7;
	font_awesome_table["save"] = 0xf0c7;
	font_awesome_table["florin-sign"] = 0xe184;
	font_awesome_table["flutter"] = 0xe694;
	font_awesome_table["fly"] = 0xf417;
	font_awesome_table["folder"] = 0xf07b;
	font_awesome_table["folder-closed"] = 0xe185;
	font_awesome_table["folder-minus"] = 0xf65d;
	font_awesome_table["folder-open"] = 0xf07c;
	font_awesome_table["folder-plus"] = 0xf65e;
	font_awesome_table["folder-tree"] = 0xf802;
	font_awesome_table["font"] = 0xf031;
	font_awesome_table["font-awesome"] = 0xf2b4;
	font_awesome_table["fonticons"] = 0xf280;
	font_awesome_table["fonticons-fi"] = 0xf3a2;
	font_awesome_table["football"] = 0xf44e;
	font_awesome_table["forgejo"] = 0xe860;
	font_awesome_table["fort-awesome"] = 0xf286;
	font_awesome_table["fort-awesome-alt"] = 0xf3a3;
	font_awesome_table["forumbee"] = 0xf211;
	font_awesome_table["forward"] = 0xf04e;
	font_awesome_table["forward-fast"] = 0xf050;
	font_awesome_table["forward-step"] = 0xf051;
	font_awesome_table["foursquare"] = 0xf180;
	font_awesome_table["franc-sign"] = 0xe18f;
	font_awesome_table["free-code-camp"] = 0xf2c5;
	font_awesome_table["freebsd"] = 0xf3a4;
	font_awesome_table["frog"] = 0xf52e;
	font_awesome_table["fulcrum"] = 0xf50b;
	font_awesome_table["futbol"] = 0xf1e3;
	font_awesome_table["g"] = 0x47;
	font_awesome_table["galactic-republic"] = 0xf50c;
	font_awesome_table["galactic-senate"] = 0xf50d;
	font_awesome_table["gamepad"] = 0xf11b;
	font_awesome_table["gas-pump"] = 0xf52f;
	font_awesome_table["gauge"] = 0xf624;
	font_awesome_table["gauge-high"] = 0xf625;
	font_awesome_table["gauge-simple"] = 0xf629;
	font_awesome_table["gauge-simple-high"] = 0xf62a;
	font_awesome_table["gavel"] = 0xf0e3;
	font_awesome_table["gear"] = 0xf013;
	font_awesome_table["cog"] = 0xf013;
	font_awesome_table["gears"] = 0xf085;
	font_awesome_table["cogs"] = 0xf085;
	font_awesome_table["gem"] = 0xf3a5;
	font_awesome_table["gemini"] = 0xe849;
	font_awesome_table["genderless"] = 0xf22d;
	font_awesome_table["get-pocket"] = 0xf265;
	font_awesome_table["gg"] = 0xf260;
	font_awesome_table["gg-circle"] = 0xf261;
	font_awesome_table["ghost"] = 0xf6e2;
	font_awesome_table["gift"] = 0xf06b;
	font_awesome_table["gifts"] = 0xf79c;
	font_awesome_table["git"] = 0xf1d3;
	font_awesome_table["git-alt"] = 0xf841;
	font_awesome_table["gitee"] = 0xe863;
	font_awesome_table["github"] = 0xf09b;
	font_awesome_table["github-alt"] = 0xf113;
	font_awesome_table["gitkraken"] = 0xf3a6;
	font_awesome_table["gitlab"] = 0xf296;
	font_awesome_table["gitter"] = 0xf426;
	font_awesome_table["glass-water"] = 0xe4f4;
	font_awesome_table["glass-water-droplet"] = 0xe4f5;
	font_awesome_table["glasses"] = 0xf530;
	font_awesome_table["glide"] = 0xf2a5;
	font_awesome_table["glide-g"] = 0xf2a6;
	font_awesome_table["globaleaks"] = 0xe85d;
	font_awesome_table["globe"] = 0xf0ac;
	font_awesome_table["gofore"] = 0xf3a7;
	font_awesome_table["golang"] = 0xe40f;
	font_awesome_table["golf-ball-tee"] = 0xf450;
	font_awesome_table["goodreads"] = 0xf3a8;
	font_awesome_table["goodreads-g"] = 0xf3a9;
	font_awesome_table["google"] = 0xf1a0;
	font_awesome_table["google-drive"] = 0xf3aa;
	font_awesome_table["google-pay"] = 0xe079;
	font_awesome_table["google-play"] = 0xf3ab;
	font_awesome_table["google-plus"] = 0xf2b3;
	font_awesome_table["google-plus-g"] = 0xf0d5;
	font_awesome_table["google-scholar"] = 0xe63b;
	font_awesome_table["google-wallet"] = 0xf1ee;
	font_awesome_table["gopuram"] = 0xf664;
	font_awesome_table["graduation-cap"] = 0xf19d;
	font_awesome_table["gratipay"] = 0xf184;
	font_awesome_table["grav"] = 0xf2d6;
	font_awesome_table["greater-than"] = 0x3e;
	font_awesome_table["greater-than-equal"] = 0xf532;
	font_awesome_table["grip"] = 0xf58d;
	font_awesome_table["grip-lines"] = 0xf7a4;
	font_awesome_table["grip-lines-vertical"] = 0xf7a5;
	font_awesome_table["grip-vertical"] = 0xf58e;
	font_awesome_table["gripfire"] = 0xf3ac;
	font_awesome_table["group-arrows-rotate"] = 0xe4f6;
	font_awesome_table["grunt"] = 0xf3ad;
	font_awesome_table["guarani-sign"] = 0xe19a;
	font_awesome_table["guilded"] = 0xe07e;
	font_awesome_table["guitar"] = 0xf7a6;
	font_awesome_table["gulp"] = 0xf3ae;
	font_awesome_table["gun"] = 0xe19b;
	font_awesome_table["h"] = 0x48;
	font_awesome_table["hacker-news"] = 0xf1d4;
	font_awesome_table["hackerrank"] = 0xf5f7;
	font_awesome_table["hammer"] = 0xf6e3;
	font_awesome_table["hamsa"] = 0xf665;
	font_awesome_table["hand"] = 0xf256;
	font_awesome_table["hand-back-fist"] = 0xf255;
	font_awesome_table["hand-dots"] = 0xf461;
	font_awesome_table["hand-fist"] = 0xf6de;
	font_awesome_table["hand-holding"] = 0xf4bd;
	font_awesome_table["hand-holding-dollar"] = 0xf4c0;
	font_awesome_table["hand-holding-droplet"] = 0xf4c1;
	font_awesome_table["hand-holding-hand"] = 0xe4f7;
	font_awesome_table["hand-holding-heart"] = 0xf4be;
	font_awesome_table["hand-holding-medical"] = 0xe05c;
	font_awesome_table["hand-lizard"] = 0xf258;
	font_awesome_table["hand-middle-finger"] = 0xf806;
	font_awesome_table["hand-peace"] = 0xf25b;
	font_awesome_table["hand-point-down"] = 0xf0a7;
	font_awesome_table["hand-point-left"] = 0xf0a5;
	font_awesome_table["hand-point-right"] = 0xf0a4;
	font_awesome_table["hand-point-up"] = 0xf0a6;
	font_awesome_table["hand-pointer"] = 0xf25a;
	font_awesome_table["hand-scissors"] = 0xf257;
	font_awesome_table["hand-sparkles"] = 0xe05d;
	font_awesome_table["hand-spock"] = 0xf259;
	font_awesome_table["handcuffs"] = 0xe4f8;
	font_awesome_table["hands"] = 0xf2a7;
	font_awesome_table["hands-asl-interpreting"] = 0xf2a3;
	font_awesome_table["hands-bound"] = 0xe4f9;
	font_awesome_table["hands-bubbles"] = 0xe05e;
	font_awesome_table["hands-clapping"] = 0xe1a8;
	font_awesome_table["hands-holding"] = 0xf4c2;
	font_awesome_table["hands-holding-child"] = 0xe4fa;
	font_awesome_table["hands-holding-circle"] = 0xe4fb;
	font_awesome_table["hands-praying"] = 0xf684;
	font_awesome_table["handshake"] = 0xf2b5;
	font_awesome_table["handshake-angle"] = 0xf4c4;
	font_awesome_table["handshake-slash"] = 0xe060;
	font_awesome_table["hanukiah"] = 0xf6e6;
	font_awesome_table["hard-drive"] = 0xf0a0;
	font_awesome_table["hashnode"] = 0xe499;
	font_awesome_table["hashtag"] = 0x23;
	font_awesome_table["hat-cowboy"] = 0xf8c0;
	font_awesome_table["hat-cowboy-side"] = 0xf8c1;
	font_awesome_table["hat-wizard"] = 0xf6e8;
	font_awesome_table["head-side-cough"] = 0xe061;
	font_awesome_table["head-side-cough-slash"] = 0xe062;
	font_awesome_table["head-side-mask"] = 0xe063;
	font_awesome_table["head-side-virus"] = 0xe064;
	font_awesome_table["heading"] = 0xf1dc;
	font_awesome_table["headphones"] = 0xf025;
	font_awesome_table["headset"] = 0xf590;
	font_awesome_table["heart"] = 0xf004;
	font_awesome_table["heart-circle-bolt"] = 0xe4fc;
	font_awesome_table["heart-circle-check"] = 0xe4fd;
	font_awesome_table["heart-circle-exclamation"] = 0xe4fe;
	font_awesome_table["heart-circle-minus"] = 0xe4ff;
	font_awesome_table["heart-circle-plus"] = 0xe500;
	font_awesome_table["heart-circle-xmark"] = 0xe501;
	font_awesome_table["heart-crack"] = 0xf7a9;
	font_awesome_table["heart-pulse"] = 0xf21e;
	font_awesome_table["helicopter"] = 0xf533;
	font_awesome_table["helicopter-symbol"] = 0xe502;
	font_awesome_table["helmet-safety"] = 0xf807;
	font_awesome_table["helmet-un"] = 0xe503;
	font_awesome_table["hexagon"] = 0xf312;
	font_awesome_table["hexagon-nodes"] = 0xe699;
	font_awesome_table["hexagon-nodes-bolt"] = 0xe69a;
	font_awesome_table["highlighter"] = 0xf591;
	font_awesome_table["hill-avalanche"] = 0xe507;
	font_awesome_table["hill-rockslide"] = 0xe508;
	font_awesome_table["hippo"] = 0xf6ed;
	font_awesome_table["hips"] = 0xf452;
	font_awesome_table["hire-a-helper"] = 0xf3b0;
	font_awesome_table["hive"] = 0xe07f;
	font_awesome_table["hockey-puck"] = 0xf453;
	font_awesome_table["holly-berry"] = 0xf7aa;
	font_awesome_table["hooli"] = 0xf427;
	font_awesome_table["hornbill"] = 0xf592;
	font_awesome_table["horse"] = 0xf6f0;
	font_awesome_table["horse-head"] = 0xf7ab;
	font_awesome_table["hospital"] = 0xf0f8;
	font_awesome_table["hospital-user"] = 0xf80d;
	font_awesome_table["hot-tub-person"] = 0xf593;
	font_awesome_table["hotdog"] = 0xf80f;
	font_awesome_table["hotel"] = 0xf594;
	font_awesome_table["hotjar"] = 0xf3b1;
	font_awesome_table["hourglass"] = 0xf254;
	font_awesome_table["hourglass-end"] = 0xf253;
	font_awesome_table["hourglass-half"] = 0xf252;
	font_awesome_table["hourglass-start"] = 0xf251;
	font_awesome_table["house"] = 0xf015;
	font_awesome_table["home"] = 0xf015;
	font_awesome_table["house-chimney"] = 0xe3af;
	font_awesome_table["house-chimney-crack"] = 0xf6f1;
	font_awesome_table["house-chimney-medical"] = 0xf7f2;
	font_awesome_table["house-chimney-user"] = 0xe065;
	font_awesome_table["house-chimney-window"] = 0xe00d;
	font_awesome_table["house-circle-check"] = 0xe509;
	font_awesome_table["house-circle-exclamation"] = 0xe50a;
	font_awesome_table["house-circle-xmark"] = 0xe50b;
	font_awesome_table["house-crack"] = 0xe3b1;
	font_awesome_table["house-fire"] = 0xe50c;
	font_awesome_table["house-flag"] = 0xe50d;
	font_awesome_table["house-flood-water"] = 0xe50e;
	font_awesome_table["house-flood-water-circle-arrow-right"] = 0xe50f;
	font_awesome_table["house-laptop"] = 0xe066;
	font_awesome_table["house-lock"] = 0xe510;
	font_awesome_table["house-medical"] = 0xe3b2;
	font_awesome_table["house-medical-circle-check"] = 0xe511;
	font_awesome_table["house-medical-circle-exclamation"] = 0xe512;
	font_awesome_table["house-medical-circle-xmark"] = 0xe513;
	font_awesome_table["house-medical-flag"] = 0xe514;
	font_awesome_table["house-signal"] = 0xe012;
	font_awesome_table["house-tsunami"] = 0xe515;
	font_awesome_table["house-user"] = 0xe1b0;
	font_awesome_table["houzz"] = 0xf27c;
	font_awesome_table["hryvnia-sign"] = 0xf6f2;
	font_awesome_table["html5"] = 0xf13b;
	font_awesome_table["hubspot"] = 0xf3b2;
	font_awesome_table["hugging-face"] = 0xe869;
	font_awesome_table["hurricane"] = 0xf751;
	font_awesome_table["i"] = 0x49;
	font_awesome_table["i-cursor"] = 0xf246;
	font_awesome_table["ice-cream"] = 0xf810;
	font_awesome_table["icicles"] = 0xf7ad;
	font_awesome_table["icons"] = 0xf86d;
	font_awesome_table["id-badge"] = 0xf2c1;
	font_awesome_table["id-card"] = 0xf2c2;
	font_awesome_table["id-card-clip"] = 0xf47f;
	font_awesome_table["ideal"] = 0xe013;
	font_awesome_table["igloo"] = 0xf7ae;
	font_awesome_table["image"] = 0xf03e;
	font_awesome_table["image-portrait"] = 0xf3e0;
	font_awesome_table["images"] = 0xf302;
	font_awesome_table["imdb"] = 0xf2d8;
	font_awesome_table["inbox"] = 0xf01c;
	font_awesome_table["indent"] = 0xf03c;
	font_awesome_table["indian-rupee-sign"] = 0xe1bc;
	font_awesome_table["industry"] = 0xf275;
	font_awesome_table["infinity"] = 0xf534;
	font_awesome_table["info"] = 0xf129;
	font_awesome_table["instagram"] = 0xf16d;
	font_awesome_table["instalod"] = 0xe081;
	font_awesome_table["intercom"] = 0xf7af;
	font_awesome_table["internet-explorer"] = 0xf26b;
	font_awesome_table["invision"] = 0xf7b0;
	font_awesome_table["ioxhost"] = 0xf208;
	font_awesome_table["italic"] = 0xf033;
	font_awesome_table["itch-io"] = 0xf83a;
	font_awesome_table["itunes"] = 0xf3b4;
	font_awesome_table["itunes-note"] = 0xf3b5;
	font_awesome_table["j"] = 0x4a;
	font_awesome_table["jar"] = 0xe516;
	font_awesome_table["jar-wheat"] = 0xe517;
	font_awesome_table["java"] = 0xf4e4;
	font_awesome_table["jedi"] = 0xf669;
	font_awesome_table["jedi-order"] = 0xf50e;
	font_awesome_table["jenkins"] = 0xf3b6;
	font_awesome_table["jet-fighter"] = 0xf0fb;
	font_awesome_table["jet-fighter-up"] = 0xe518;
	font_awesome_table["jira"] = 0xf7b1;
	font_awesome_table["joget"] = 0xf3b7;
	font_awesome_table["joint"] = 0xf595;
	font_awesome_table["joomla"] = 0xf1aa;
	font_awesome_table["js"] = 0xf3b8;
	font_awesome_table["jsfiddle"] = 0xf1cc;
	font_awesome_table["jug-detergent"] = 0xe519;
	font_awesome_table["julia"] = 0xe852;
	font_awesome_table["jxl"] = 0xe67b;
	font_awesome_table["k"] = 0x4b;
	font_awesome_table["kaaba"] = 0xf66b;
	font_awesome_table["kaggle"] = 0xf5fa;
	font_awesome_table["kakao-talk"] = 0xe7d7;
	font_awesome_table["key"] = 0xf084;
	font_awesome_table["keybase"] = 0xf4f5;
	font_awesome_table["keyboard"] = 0xf11c;
	font_awesome_table["keycdn"] = 0xf3ba;
	font_awesome_table["khanda"] = 0xf66d;
	font_awesome_table["kickstarter"] = 0xf3bb;
	font_awesome_table["kickstarter-k"] = 0xf3bc;
	font_awesome_table["kip-sign"] = 0xe1c4;
	font_awesome_table["kit-medical"] = 0xf479;
	font_awesome_table["kitchen-set"] = 0xe51a;
	font_awesome_table["kiwi-bird"] = 0xf535;
	font_awesome_table["ko-fi"] = 0xe856;
	font_awesome_table["korvue"] = 0xf42f;
	font_awesome_table["kubernetes"] = 0xe857;
	font_awesome_table["l"] = 0x4c;
	font_awesome_table["land-mine-on"] = 0xe51b;
	font_awesome_table["landmark"] = 0xf66f;
	font_awesome_table["landmark-dome"] = 0xf752;
	font_awesome_table["landmark-flag"] = 0xe51c;
	font_awesome_table["language"] = 0xf1ab;
	font_awesome_table["laptop"] = 0xf109;
	font_awesome_table["laptop-code"] = 0xf5fc;
	font_awesome_table["laptop-file"] = 0xe51d;
	font_awesome_table["laptop-medical"] = 0xf812;
	font_awesome_table["laravel"] = 0xf3bd;
	font_awesome_table["lari-sign"] = 0xe1c8;
	font_awesome_table["lastfm"] = 0xf202;
	font_awesome_table["layer-group"] = 0xf5fd;
	font_awesome_table["leaf"] = 0xf06c;
	font_awesome_table["leanpub"] = 0xf212;
	font_awesome_table["leetcode"] = 0xe86a;
	font_awesome_table["left-long"] = 0xf30a;
	font_awesome_table["left-right"] = 0xf337;
	font_awesome_table["lemon"] = 0xf094;
	font_awesome_table["leo"] = 0xe84a;
	font_awesome_table["less"] = 0xf41d;
	font_awesome_table["less-than"] = 0x3c;
	font_awesome_table["less-than-equal"] = 0xf537;
	font_awesome_table["letterboxd"] = 0xe62d;
	font_awesome_table["libra"] = 0xe84b;
	font_awesome_table["life-ring"] = 0xf1cd;
	font_awesome_table["lightbulb"] = 0xf0eb;
	font_awesome_table["line"] = 0xf3c0;
	font_awesome_table["lines-leaning"] = 0xe51e;
	font_awesome_table["link"] = 0xf0c1;
	font_awesome_table["link-slash"] = 0xf127;
	font_awesome_table["linkedin"] = 0xf08c;
	font_awesome_table["linkedin-in"] = 0xf0e1;
	font_awesome_table["linktree"] = 0xe7d8;
	font_awesome_table["linode"] = 0xf2b8;
	font_awesome_table["linux"] = 0xf17c;
	font_awesome_table["lira-sign"] = 0xf195;
	font_awesome_table["list"] = 0xf03a;
	font_awesome_table["list-check"] = 0xf0ae;
	font_awesome_table["list-ol"] = 0xf0cb;
	font_awesome_table["list-ul"] = 0xf0ca;
	font_awesome_table["litecoin-sign"] = 0xe1d3;
	font_awesome_table["location-arrow"] = 0xf124;
	font_awesome_table["location-crosshairs"] = 0xf601;
	font_awesome_table["location-dot"] = 0xf3c5;
	font_awesome_table["location-pin"] = 0xf041;
	font_awesome_table["location-pin-lock"] = 0xe51f;
	font_awesome_table["lock"] = 0xf023;
	font_awesome_table["lock-open"] = 0xf3c1;
	font_awesome_table["locust"] = 0xe520;
	font_awesome_table["lumon"] = 0xe7e2;
	font_awesome_table["lumon-drop"] = 0xe7e3;
	font_awesome_table["lungs"] = 0xf604;
	font_awesome_table["lungs-virus"] = 0xe067;
	font_awesome_table["lyft"] = 0xf3c3;
	font_awesome_table["m"] = 0x4d;
	font_awesome_table["magento"] = 0xf3c4;
	font_awesome_table["magnet"] = 0xf076;
	font_awesome_table["magnifying-glass"] = 0xf002;
	font_awesome_table["magnifying-glass-arrow-right"] = 0xe521;
	font_awesome_table["magnifying-glass-chart"] = 0xe522;
	font_awesome_table["magnifying-glass-dollar"] = 0xf688;
	font_awesome_table["magnifying-glass-location"] = 0xf689;
	font_awesome_table["magnifying-glass-minus"] = 0xf010;
	font_awesome_table["magnifying-glass-plus"] = 0xf00e;
	font_awesome_table["mailchimp"] = 0xf59e;
	font_awesome_table["manat-sign"] = 0xe1d5;
	font_awesome_table["mandalorian"] = 0xf50f;
	font_awesome_table["map"] = 0xf279;
	font_awesome_table["map-location"] = 0xf59f;
	font_awesome_table["map-location-dot"] = 0xf5a0;
	font_awesome_table["map-pin"] = 0xf276;
	font_awesome_table["markdown"] = 0xf60f;
	font_awesome_table["marker"] = 0xf5a1;
	font_awesome_table["mars"] = 0xf222;
	font_awesome_table["mars-and-venus"] = 0xf224;
	font_awesome_table["mars-and-venus-burst"] = 0xe523;
	font_awesome_table["mars-double"] = 0xf227;
	font_awesome_table["mars-stroke"] = 0xf229;
	font_awesome_table["mars-stroke-right"] = 0xf22b;
	font_awesome_table["mars-stroke-up"] = 0xf22a;
	font_awesome_table["martini-glass"] = 0xf57b;
	font_awesome_table["martini-glass-citrus"] = 0xf561;
	font_awesome_table["martini-glass-empty"] = 0xf000;
	font_awesome_table["mask"] = 0xf6fa;
	font_awesome_table["mask-face"] = 0xe1d7;
	font_awesome_table["mask-ventilator"] = 0xe524;
	font_awesome_table["masks-theater"] = 0xf630;
	font_awesome_table["mastodon"] = 0xf4f6;
	font_awesome_table["mattress-pillow"] = 0xe525;
	font_awesome_table["maxcdn"] = 0xf136;
	font_awesome_table["maximize"] = 0xf31e;
	font_awesome_table["mdb"] = 0xf8ca;
	font_awesome_table["medal"] = 0xf5a2;
	font_awesome_table["medapps"] = 0xf3c6;
	font_awesome_table["medium"] = 0xf23a;
	font_awesome_table["medrt"] = 0xf3c8;
	font_awesome_table["meetup"] = 0xf2e0;
	font_awesome_table["megaport"] = 0xf5a3;
	font_awesome_table["memory"] = 0xf538;
	font_awesome_table["mendeley"] = 0xf7b3;
	font_awesome_table["menorah"] = 0xf676;
	font_awesome_table["mercury"] = 0xf223;
	font_awesome_table["message"] = 0xf27a;
	font_awesome_table["meta"] = 0xe49b;
	font_awesome_table["meteor"] = 0xf753;
	font_awesome_table["microblog"] = 0xe01a;
	font_awesome_table["microchip"] = 0xf2db;
	font_awesome_table["microphone"] = 0xf130;
	font_awesome_table["microphone-lines"] = 0xf3c9;
	font_awesome_table["microphone-lines-slash"] = 0xf539;
	font_awesome_table["microphone-slash"] = 0xf131;
	font_awesome_table["microscope"] = 0xf610;
	font_awesome_table["microsoft"] = 0xf3ca;
	font_awesome_table["mill-sign"] = 0xe1ed;
	font_awesome_table["minimize"] = 0xf78c;
	font_awesome_table["mintbit"] = 0xe62f;
	font_awesome_table["minus"] = 0xf068;
	font_awesome_table["mitten"] = 0xf7b5;
	font_awesome_table["mix"] = 0xf3cb;
	font_awesome_table["mixcloud"] = 0xf289;
	font_awesome_table["mixer"] = 0xe056;
	font_awesome_table["mizuni"] = 0xf3cc;
	font_awesome_table["mobile"] = 0xf3ce;
	font_awesome_table["mobile-button"] = 0xf10b;
	font_awesome_table["mobile-retro"] = 0xe527;
	font_awesome_table["mobile-screen"] = 0xf3cf;
	font_awesome_table["mobile-screen-button"] = 0xf3cd;
	font_awesome_table["mobile-vibrate"] = 0xe816;
	font_awesome_table["modx"] = 0xf285;
	font_awesome_table["monero"] = 0xf3d0;
	font_awesome_table["money-bill"] = 0xf0d6;
	font_awesome_table["money-bill-1"] = 0xf3d1;
	font_awesome_table["money-bill-1-wave"] = 0xf53b;
	font_awesome_table["money-bill-transfer"] = 0xe528;
	font_awesome_table["money-bill-trend-up"] = 0xe529;
	font_awesome_table["money-bill-wave"] = 0xf53a;
	font_awesome_table["money-bill-wheat"] = 0xe52a;
	font_awesome_table["money-bills"] = 0xe1f3;
	font_awesome_table["money-check"] = 0xf53c;
	font_awesome_table["money-check-dollar"] = 0xf53d;
	font_awesome_table["monument"] = 0xf5a6;
	font_awesome_table["moon"] = 0xf186;
	font_awesome_table["mortar-pestle"] = 0xf5a7;
	font_awesome_table["mosque"] = 0xf678;
	font_awesome_table["mosquito"] = 0xe52b;
	font_awesome_table["mosquito-net"] = 0xe52c;
	font_awesome_table["motorcycle"] = 0xf21c;
	font_awesome_table["mound"] = 0xe52d;
	font_awesome_table["mountain"] = 0xf6fc;
	font_awesome_table["mountain-city"] = 0xe52e;
	font_awesome_table["mountain-sun"] = 0xe52f;
	font_awesome_table["mug-hot"] = 0xf7b6;
	font_awesome_table["mug-saucer"] = 0xf0f4;
	font_awesome_table["music"] = 0xf001;
	font_awesome_table["'n'"] = 0x4e;
	font_awesome_table["naira-sign"] = 0xe1f6;
	font_awesome_table["napster"] = 0xf3d2;
	font_awesome_table["neos"] = 0xf612;
	font_awesome_table["network-wired"] = 0xf6ff;
	font_awesome_table["neuter"] = 0xf22c;
	font_awesome_table["newspaper"] = 0xf1ea;
	font_awesome_table["nfc-directional"] = 0xe530;
	font_awesome_table["nfc-symbol"] = 0xe531;
	font_awesome_table["nimblr"] = 0xf5a8;
	font_awesome_table["node"] = 0xf419;
	font_awesome_table["node-js"] = 0xf3d3;
	font_awesome_table["non-binary"] = 0xe807;
	font_awesome_table["not-equal"] = 0xf53e;
	font_awesome_table["notdef"] = 0xe1fe;
	font_awesome_table["note-sticky"] = 0xf249;
	font_awesome_table["notes-medical"] = 0xf481;
	font_awesome_table["notion"] = 0xe7d9;
	font_awesome_table["npm"] = 0xf3d4;
	font_awesome_table["ns8"] = 0xf3d5;
	font_awesome_table["nutritionix"] = 0xf3d6;
	font_awesome_table["o"] = 0x4f;
	font_awesome_table["object-group"] = 0xf247;
	font_awesome_table["object-ungroup"] = 0xf248;
	font_awesome_table["obsidian"] = 0xe879;
	font_awesome_table["octagon"] = 0xf306;
	font_awesome_table["octopus-deploy"] = 0xe082;
	font_awesome_table["odnoklassniki"] = 0xf263;
	font_awesome_table["odysee"] = 0xe5c6;
	font_awesome_table["oil-can"] = 0xf613;
	font_awesome_table["oil-well"] = 0xe532;
	font_awesome_table["old-republic"] = 0xf510;
	font_awesome_table["om"] = 0xf679;
	font_awesome_table["openai"] = 0xe7cf;
	font_awesome_table["opencart"] = 0xf23d;
	font_awesome_table["openid"] = 0xf19b;
	font_awesome_table["openstreetmap"] = 0xe86b;
	font_awesome_table["opensuse"] = 0xe62b;
	font_awesome_table["opera"] = 0xf26a;
	font_awesome_table["optin-monster"] = 0xf23c;
	font_awesome_table["orcid"] = 0xf8d2;
	font_awesome_table["osi"] = 0xf41a;
	font_awesome_table["otter"] = 0xf700;
	font_awesome_table["outdent"] = 0xf03b;
	font_awesome_table["p"] = 0x50;
	font_awesome_table["padlet"] = 0xe4a0;
	font_awesome_table["page4"] = 0xf3d7;
	font_awesome_table["pagelines"] = 0xf18c;
	font_awesome_table["pager"] = 0xf815;
	font_awesome_table["paint-roller"] = 0xf5aa;
	font_awesome_table["paintbrush"] = 0xf1fc;
	font_awesome_table["palette"] = 0xf53f;
	font_awesome_table["palfed"] = 0xf3d8;
	font_awesome_table["pallet"] = 0xf482;
	font_awesome_table["pandora"] = 0xe7da;
	font_awesome_table["panorama"] = 0xe209;
	font_awesome_table["paper-plane"] = 0xf1d8;
	font_awesome_table["paperclip"] = 0xf0c6;
	font_awesome_table["parachute-box"] = 0xf4cd;
	font_awesome_table["paragraph"] = 0xf1dd;
	font_awesome_table["passport"] = 0xf5ab;
	font_awesome_table["paste"] = 0xf0ea;
	font_awesome_table["patreon"] = 0xf3d9;
	font_awesome_table["pause"] = 0xf04c;
	font_awesome_table["paw"] = 0xf1b0;
	font_awesome_table["paypal"] = 0xf1ed;
	font_awesome_table["peace"] = 0xf67c;
	font_awesome_table["pen"] = 0xf304;
	font_awesome_table["pen-clip"] = 0xf305;
	font_awesome_table["pen-fancy"] = 0xf5ac;
	font_awesome_table["pen-nib"] = 0xf5ad;
	font_awesome_table["pen-ruler"] = 0xf5ae;
	font_awesome_table["pen-to-square"] = 0xf044;
	font_awesome_table["edit"] = 0xf044;
	font_awesome_table["pencil"] = 0xf303;
	font_awesome_table["pentagon"] = 0xe790;
	font_awesome_table["people-arrows"] = 0xe068;
	font_awesome_table["people-carry-box"] = 0xf4ce;
	font_awesome_table["people-group"] = 0xe533;
	font_awesome_table["people-line"] = 0xe534;
	font_awesome_table["people-pulling"] = 0xe535;
	font_awesome_table["people-robbery"] = 0xe536;
	font_awesome_table["people-roof"] = 0xe537;
	font_awesome_table["pepper-hot"] = 0xf816;
	font_awesome_table["perbyte"] = 0xe083;
	font_awesome_table["percent"] = 0x25;
	font_awesome_table["periscope"] = 0xf3da;
	font_awesome_table["person"] = 0xf183;
	font_awesome_table["person-arrow-down-to-line"] = 0xe538;
	font_awesome_table["person-arrow-up-from-line"] = 0xe539;
	font_awesome_table["person-biking"] = 0xf84a;
	font_awesome_table["person-booth"] = 0xf756;
	font_awesome_table["person-breastfeeding"] = 0xe53a;
	font_awesome_table["person-burst"] = 0xe53b;
	font_awesome_table["person-cane"] = 0xe53c;
	font_awesome_table["person-chalkboard"] = 0xe53d;
	font_awesome_table["person-circle-check"] = 0xe53e;
	font_awesome_table["person-circle-exclamation"] = 0xe53f;
	font_awesome_table["person-circle-minus"] = 0xe540;
	font_awesome_table["person-circle-plus"] = 0xe541;
	font_awesome_table["person-circle-question"] = 0xe542;
	font_awesome_table["person-circle-xmark"] = 0xe543;
	font_awesome_table["person-digging"] = 0xf85e;
	font_awesome_table["person-dots-from-line"] = 0xf470;
	font_awesome_table["person-dress"] = 0xf182;
	font_awesome_table["person-dress-burst"] = 0xe544;
	font_awesome_table["person-drowning"] = 0xe545;
	font_awesome_table["person-falling"] = 0xe546;
	font_awesome_table["person-falling-burst"] = 0xe547;
	font_awesome_table["person-half-dress"] = 0xe548;
	font_awesome_table["person-harassing"] = 0xe549;
	font_awesome_table["person-hiking"] = 0xf6ec;
	font_awesome_table["person-military-pointing"] = 0xe54a;
	font_awesome_table["person-military-rifle"] = 0xe54b;
	font_awesome_table["person-military-to-person"] = 0xe54c;
	font_awesome_table["person-praying"] = 0xf683;
	font_awesome_table["person-pregnant"] = 0xe31e;
	font_awesome_table["person-rays"] = 0xe54d;
	font_awesome_table["person-rifle"] = 0xe54e;
	font_awesome_table["person-running"] = 0xf70c;
	font_awesome_table["person-shelter"] = 0xe54f;
	font_awesome_table["person-skating"] = 0xf7c5;
	font_awesome_table["person-skiing"] = 0xf7c9;
	font_awesome_table["person-skiing-nordic"] = 0xf7ca;
	font_awesome_table["person-snowboarding"] = 0xf7ce;
	font_awesome_table["person-swimming"] = 0xf5c4;
	font_awesome_table["person-through-window"] = 0xe5a9;
	font_awesome_table["person-walking"] = 0xf554;
	font_awesome_table["person-walking-arrow-loop-left"] = 0xe551;
	font_awesome_table["person-walking-arrow-right"] = 0xe552;
	font_awesome_table["person-walking-dashed-line-arrow-right"] = 0xe553;
	font_awesome_table["person-walking-luggage"] = 0xe554;
	font_awesome_table["person-walking-with-cane"] = 0xf29d;
	font_awesome_table["peseta-sign"] = 0xe221;
	font_awesome_table["peso-sign"] = 0xe222;
	font_awesome_table["phabricator"] = 0xf3db;
	font_awesome_table["phoenix-framework"] = 0xf3dc;
	font_awesome_table["phoenix-squadron"] = 0xf511;
	font_awesome_table["phone"] = 0xf095;
	font_awesome_table["phone-flip"] = 0xf879;
	font_awesome_table["phone-slash"] = 0xf3dd;
	font_awesome_table["phone-volume"] = 0xf2a0;
	font_awesome_table["photo-film"] = 0xf87c;
	font_awesome_table["php"] = 0xf457;
	font_awesome_table["picture-in-picture"] = 0xe80b;
	font_awesome_table["pied-piper"] = 0xf2ae;
	font_awesome_table["pied-piper-alt"] = 0xf1a8;
	font_awesome_table["pied-piper-hat"] = 0xf4e5;
	font_awesome_table["pied-piper-pp"] = 0xf1a7;
	font_awesome_table["piggy-bank"] = 0xf4d3;
	font_awesome_table["pills"] = 0xf484;
	font_awesome_table["pinterest"] = 0xf0d2;
	font_awesome_table["pinterest-p"] = 0xf231;
	font_awesome_table["pisces"] = 0xe84c;
	font_awesome_table["pix"] = 0xe43a;
	font_awesome_table["pixelfed"] = 0xe7db;
	font_awesome_table["pixiv"] = 0xe640;
	font_awesome_table["pizza-slice"] = 0xf818;
	font_awesome_table["place-of-worship"] = 0xf67f;
	font_awesome_table["plane"] = 0xf072;
	font_awesome_table["plane-arrival"] = 0xf5af;
	font_awesome_table["plane-circle-check"] = 0xe555;
	font_awesome_table["plane-circle-exclamation"] = 0xe556;
	font_awesome_table["plane-circle-xmark"] = 0xe557;
	font_awesome_table["plane-departure"] = 0xf5b0;
	font_awesome_table["plane-lock"] = 0xe558;
	font_awesome_table["plane-slash"] = 0xe069;
	font_awesome_table["plane-up"] = 0xe22d;
	font_awesome_table["plant-wilt"] = 0xe5aa;
	font_awesome_table["plate-wheat"] = 0xe55a;
	font_awesome_table["play"] = 0xf04b;
	font_awesome_table["playstation"] = 0xf3df;
	font_awesome_table["plug"] = 0xf1e6;
	font_awesome_table["plug-circle-bolt"] = 0xe55b;
	font_awesome_table["plug-circle-check"] = 0xe55c;
	font_awesome_table["plug-circle-exclamation"] = 0xe55d;
	font_awesome_table["plug-circle-minus"] = 0xe55e;
	font_awesome_table["plug-circle-plus"] = 0xe55f;
	font_awesome_table["plug-circle-xmark"] = 0xe560;
	font_awesome_table["plus"] = 0x2b;
	font_awesome_table["plus-minus"] = 0xe43c;
	font_awesome_table["podcast"] = 0xf2ce;
	font_awesome_table["poo"] = 0xf2fe;
	font_awesome_table["poo-storm"] = 0xf75a;
	font_awesome_table["poop"] = 0xf619;
	font_awesome_table["postgresql"] = 0xe858;
	font_awesome_table["power-off"] = 0xf011;
	font_awesome_table["prescription"] = 0xf5b1;
	font_awesome_table["prescription-bottle"] = 0xf485;
	font_awesome_table["prescription-bottle-medical"] = 0xf486;
	font_awesome_table["print"] = 0xf02f;
	font_awesome_table["product-hunt"] = 0xf288;
	font_awesome_table["pump-medical"] = 0xe06a;
	font_awesome_table["pump-soap"] = 0xe06b;
	font_awesome_table["pushed"] = 0xf3e1;
	font_awesome_table["puzzle-piece"] = 0xf12e;
	font_awesome_table["python"] = 0xf3e2;
	font_awesome_table["q"] = 0x51;
	font_awesome_table["qq"] = 0xf1d6;
	font_awesome_table["qrcode"] = 0xf029;
	font_awesome_table["question"] = 0x3f;
	font_awesome_table["quinscape"] = 0xf459;
	font_awesome_table["quora"] = 0xf2c4;
	font_awesome_table["quote-left"] = 0xf10d;
	font_awesome_table["quote-right"] = 0xf10e;
	font_awesome_table["r"] = 0x52;
	font_awesome_table["r-project"] = 0xf4f7;
	font_awesome_table["radiation"] = 0xf7b9;
	font_awesome_table["radio"] = 0xf8d7;
	font_awesome_table["rainbow"] = 0xf75b;
	font_awesome_table["ranking-star"] = 0xe561;
	font_awesome_table["raspberry-pi"] = 0xf7bb;
	font_awesome_table["ravelry"] = 0xf2d9;
	font_awesome_table["react"] = 0xf41b;
	font_awesome_table["reacteurope"] = 0xf75d;
	font_awesome_table["readme"] = 0xf4d5;
	font_awesome_table["rebel"] = 0xf1d0;
	font_awesome_table["receipt"] = 0xf543;
	font_awesome_table["record-vinyl"] = 0xf8d9;
	font_awesome_table["rectangle-ad"] = 0xf641;
	font_awesome_table["rectangle-list"] = 0xf022;
	font_awesome_table["rectangle-xmark"] = 0xf410;
	font_awesome_table["window-close"] = 0xf410;
	font_awesome_table["recycle"] = 0xf1b8;
	font_awesome_table["red-river"] = 0xf3e3;
	font_awesome_table["reddit"] = 0xf1a1;
	font_awesome_table["reddit-alien"] = 0xf281;
	font_awesome_table["redhat"] = 0xf7bc;
	font_awesome_table["registered"] = 0xf25d;
	font_awesome_table["renren"] = 0xf18b;
	font_awesome_table["repeat"] = 0xf363;
	font_awesome_table["reply"] = 0xf3e5;
	font_awesome_table["reply-all"] = 0xf122;
	font_awesome_table["replyd"] = 0xf3e6;
	font_awesome_table["republican"] = 0xf75e;
	font_awesome_table["researchgate"] = 0xf4f8;
	font_awesome_table["resolving"] = 0xf3e7;
	font_awesome_table["restroom"] = 0xf7bd;
	font_awesome_table["retweet"] = 0xf079;
	font_awesome_table["rev"] = 0xf5b2;
	font_awesome_table["ribbon"] = 0xf4d6;
	font_awesome_table["right-from-bracket"] = 0xf2f5;
	font_awesome_table["right-left"] = 0xf362;
	font_awesome_table["right-long"] = 0xf30b;
	font_awesome_table["right-to-bracket"] = 0xf2f6;
	font_awesome_table["ring"] = 0xf70b;
	font_awesome_table["road"] = 0xf018;
	font_awesome_table["road-barrier"] = 0xe562;
	font_awesome_table["road-bridge"] = 0xe563;
	font_awesome_table["road-circle-check"] = 0xe564;
	font_awesome_table["road-circle-exclamation"] = 0xe565;
	font_awesome_table["road-circle-xmark"] = 0xe566;
	font_awesome_table["road-lock"] = 0xe567;
	font_awesome_table["road-spikes"] = 0xe568;
	font_awesome_table["robot"] = 0xf544;
	font_awesome_table["rocket"] = 0xf135;
	font_awesome_table["rocketchat"] = 0xf3e8;
	font_awesome_table["rockrms"] = 0xf3e9;
	font_awesome_table["rotate"] = 0xf2f1;
	font_awesome_table["sync-alt"] = 0xf2f1;
	font_awesome_table["rotate-left"] = 0xf2ea;
	font_awesome_table["rotate-right"] = 0xf2f9;
	font_awesome_table["route"] = 0xf4d7;
	font_awesome_table["rss"] = 0xf09e;
	font_awesome_table["ruble-sign"] = 0xf158;
	font_awesome_table["rug"] = 0xe569;
	font_awesome_table["ruler"] = 0xf545;
	font_awesome_table["ruler-combined"] = 0xf546;
	font_awesome_table["ruler-horizontal"] = 0xf547;
	font_awesome_table["ruler-vertical"] = 0xf548;
	font_awesome_table["rupee-sign"] = 0xf156;
	font_awesome_table["rupiah-sign"] = 0xe23d;
	font_awesome_table["rust"] = 0xe07a;
	font_awesome_table["s"] = 0x53;
	font_awesome_table["sack-dollar"] = 0xf81d;
	font_awesome_table["sack-xmark"] = 0xe56a;
	font_awesome_table["safari"] = 0xf267;
	font_awesome_table["sagittarius"] = 0xe84d;
	font_awesome_table["sailboat"] = 0xe445;
	font_awesome_table["salesforce"] = 0xf83b;
	font_awesome_table["sass"] = 0xf41e;
	font_awesome_table["satellite"] = 0xf7bf;
	font_awesome_table["satellite-dish"] = 0xf7c0;
	font_awesome_table["scale-balanced"] = 0xf24e;
	font_awesome_table["scale-unbalanced"] = 0xf515;
	font_awesome_table["scale-unbalanced-flip"] = 0xf516;
	font_awesome_table["scaleway"] = 0xe859;
	font_awesome_table["schlix"] = 0xf3ea;
	font_awesome_table["school"] = 0xf549;
	font_awesome_table["school-circle-check"] = 0xe56b;
	font_awesome_table["school-circle-exclamation"] = 0xe56c;
	font_awesome_table["school-circle-xmark"] = 0xe56d;
	font_awesome_table["school-flag"] = 0xe56e;
	font_awesome_table["school-lock"] = 0xe56f;
	font_awesome_table["scissors"] = 0xf0c4;
	font_awesome_table["scorpio"] = 0xe84e;
	font_awesome_table["screenpal"] = 0xe570;
	font_awesome_table["screwdriver"] = 0xf54a;
	font_awesome_table["screwdriver-wrench"] = 0xf7d9;
	font_awesome_table["scribd"] = 0xf28a;
	font_awesome_table["scroll"] = 0xf70e;
	font_awesome_table["scroll-torah"] = 0xf6a0;
	font_awesome_table["sd-card"] = 0xf7c2;
	font_awesome_table["searchengin"] = 0xf3eb;
	font_awesome_table["section"] = 0xe447;
	font_awesome_table["seedling"] = 0xf4d8;
	font_awesome_table["sellcast"] = 0xf2da;
	font_awesome_table["sellsy"] = 0xf213;
	font_awesome_table["septagon"] = 0xe820;
	font_awesome_table["server"] = 0xf233;
	font_awesome_table["servicestack"] = 0xf3ec;
	font_awesome_table["shapes"] = 0xf61f;
	font_awesome_table["share"] = 0xf064;
	font_awesome_table["share-from-square"] = 0xf14d;
	font_awesome_table["share-nodes"] = 0xf1e0;
	font_awesome_table["sheet-plastic"] = 0xe571;
	font_awesome_table["shekel-sign"] = 0xf20b;
	font_awesome_table["shield"] = 0xf132;
	font_awesome_table["shield-cat"] = 0xe572;
	font_awesome_table["shield-dog"] = 0xe573;
	font_awesome_table["shield-halved"] = 0xf3ed;
	font_awesome_table["shield-heart"] = 0xe574;
	font_awesome_table["shield-virus"] = 0xe06c;
	font_awesome_table["ship"] = 0xf21a;
	font_awesome_table["shirt"] = 0xf553;
	font_awesome_table["shirtsinbulk"] = 0xf214;
	font_awesome_table["shoe-prints"] = 0xf54b;
	font_awesome_table["shoelace"] = 0xe60c;
	font_awesome_table["shop"] = 0xf54f;
	font_awesome_table["shop-lock"] = 0xe4a5;
	font_awesome_table["shop-slash"] = 0xe070;
	font_awesome_table["shopify"] = 0xe057;
	font_awesome_table["shopware"] = 0xf5b5;
	font_awesome_table["shower"] = 0xf2cc;
	font_awesome_table["shrimp"] = 0xe448;
	font_awesome_table["shuffle"] = 0xf074;
	font_awesome_table["shuttle-space"] = 0xf197;
	font_awesome_table["sign-hanging"] = 0xf4d9;
	font_awesome_table["signal"] = 0xf012;
	font_awesome_table["signal-messenger"] = 0xe663;
	font_awesome_table["signature"] = 0xf5b7;
	font_awesome_table["signs-post"] = 0xf277;
	font_awesome_table["sim-card"] = 0xf7c4;
	font_awesome_table["simplybuilt"] = 0xf215;
	font_awesome_table["single-quote-left"] = 0xe81b;
	font_awesome_table["single-quote-right"] = 0xe81c;
	font_awesome_table["sink"] = 0xe06d;
	font_awesome_table["sistrix"] = 0xf3ee;
	font_awesome_table["sitemap"] = 0xf0e8;
	font_awesome_table["sith"] = 0xf512;
	font_awesome_table["sitrox"] = 0xe44a;
	font_awesome_table["sketch"] = 0xf7c6;
	font_awesome_table["skull"] = 0xf54c;
	font_awesome_table["skull-crossbones"] = 0xf714;
	font_awesome_table["skyatlas"] = 0xf216;
	font_awesome_table["skype"] = 0xf17e;
	font_awesome_table["slack"] = 0xf198;
	font_awesome_table["slash"] = 0xf715;
	font_awesome_table["sleigh"] = 0xf7cc;
	font_awesome_table["sliders"] = 0xf1de;
	font_awesome_table["slideshare"] = 0xf1e7;
	font_awesome_table["smog"] = 0xf75f;
	font_awesome_table["smoking"] = 0xf48d;
	font_awesome_table["snapchat"] = 0xf2ab;
	font_awesome_table["snowflake"] = 0xf2dc;
	font_awesome_table["snowman"] = 0xf7d0;
	font_awesome_table["snowplow"] = 0xf7d2;
	font_awesome_table["soap"] = 0xe06e;
	font_awesome_table["socks"] = 0xf696;
	font_awesome_table["solana"] = 0xe85e;
	font_awesome_table["solar-panel"] = 0xf5ba;
	font_awesome_table["sort"] = 0xf0dc;
	font_awesome_table["sort-down"] = 0xf0dd;
	font_awesome_table["sort-up"] = 0xf0de;
	font_awesome_table["soundcloud"] = 0xf1be;
	font_awesome_table["sourcetree"] = 0xf7d3;
	font_awesome_table["spa"] = 0xf5bb;
	font_awesome_table["space-awesome"] = 0xe5ac;
	font_awesome_table["spaghetti-monster-flying"] = 0xf67b;
	font_awesome_table["speakap"] = 0xf3f3;
	font_awesome_table["speaker-deck"] = 0xf83c;
	font_awesome_table["spell-check"] = 0xf891;
	font_awesome_table["spider"] = 0xf717;
	font_awesome_table["spinner"] = 0xf110;
	font_awesome_table["spiral"] = 0xe80a;
	font_awesome_table["splotch"] = 0xf5bc;
	font_awesome_table["spoon"] = 0xf2e5;
	font_awesome_table["spotify"] = 0xf1bc;
	font_awesome_table["spray-can"] = 0xf5bd;
	font_awesome_table["spray-can-sparkles"] = 0xf5d0;
	font_awesome_table["square"] = 0xf0c8;
	font_awesome_table["square-arrow-up-right"] = 0xf14c;
	font_awesome_table["square-behance"] = 0xf1b5;
	font_awesome_table["square-binary"] = 0xe69b;
	font_awesome_table["square-bluesky"] = 0xe6a3;
	font_awesome_table["square-caret-down"] = 0xf150;
	font_awesome_table["square-caret-left"] = 0xf191;
	font_awesome_table["square-caret-right"] = 0xf152;
	font_awesome_table["square-caret-up"] = 0xf151;
	font_awesome_table["square-check"] = 0xf14a;
	font_awesome_table["square-deskpro"] = 0xe844;
	font_awesome_table["square-dribbble"] = 0xf397;
	font_awesome_table["square-envelope"] = 0xf199;
	font_awesome_table["square-facebook"] = 0xf082;
	font_awesome_table["square-figma"] = 0xe7e4;
	font_awesome_table["square-font-awesome"] = 0xe5ad;
	font_awesome_table["square-font-awesome-stroke"] = 0xf35c;
	font_awesome_table["square-full"] = 0xf45c;
	font_awesome_table["square-git"] = 0xf1d2;
	font_awesome_table["square-github"] = 0xf092;
	font_awesome_table["square-gitlab"] = 0xe5ae;
	font_awesome_table["square-google-plus"] = 0xf0d4;
	font_awesome_table["square-h"] = 0xf0fd;
	font_awesome_table["square-hacker-news"] = 0xf3af;
	font_awesome_table["square-instagram"] = 0xe055;
	font_awesome_table["square-js"] = 0xf3b9;
	font_awesome_table["square-lastfm"] = 0xf203;
	font_awesome_table["square-letterboxd"] = 0xe62e;
	font_awesome_table["square-linkedin"] = 0xe7d0;
	font_awesome_table["square-minus"] = 0xf146;
	font_awesome_table["square-nfi"] = 0xe576;
	font_awesome_table["square-odnoklassniki"] = 0xf264;
	font_awesome_table["square-parking"] = 0xf540;
	font_awesome_table["square-pen"] = 0xf14b;
	font_awesome_table["square-person-confined"] = 0xe577;
	font_awesome_table["square-phone"] = 0xf098;
	font_awesome_table["square-phone-flip"] = 0xf87b;
	font_awesome_table["square-pied-piper"] = 0xe01e;
	font_awesome_table["square-pinterest"] = 0xf0d3;
	font_awesome_table["square-plus"] = 0xf0fe;
	font_awesome_table["square-poll-horizontal"] = 0xf682;
	font_awesome_table["square-poll-vertical"] = 0xf681;
	font_awesome_table["square-reddit"] = 0xf1a2;
	font_awesome_table["square-root-variable"] = 0xf698;
	font_awesome_table["square-rss"] = 0xf143;
	font_awesome_table["square-share-nodes"] = 0xf1e1;
	font_awesome_table["square-snapchat"] = 0xf2ad;
	font_awesome_table["square-steam"] = 0xf1b7;
	font_awesome_table["square-threads"] = 0xe619;
	font_awesome_table["square-tumblr"] = 0xf174;
	font_awesome_table["square-twitter"] = 0xf081;
	font_awesome_table["square-up-right"] = 0xf360;
	font_awesome_table["square-upwork"] = 0xe67c;
	font_awesome_table["square-viadeo"] = 0xf2aa;
	font_awesome_table["square-vimeo"] = 0xf194;
	font_awesome_table["square-virus"] = 0xe578;
	font_awesome_table["square-web-awesome"] = 0xe683;
	font_awesome_table["square-web-awesome-stroke"] = 0xe684;
	font_awesome_table["square-whatsapp"] = 0xf40c;
	font_awesome_table["square-x-twitter"] = 0xe61a;
	font_awesome_table["square-xing"] = 0xf169;
	font_awesome_table["square-xmark"] = 0xf2d3;
	font_awesome_table["square-youtube"] = 0xf431;
	font_awesome_table["squarespace"] = 0xf5be;
	font_awesome_table["stack-exchange"] = 0xf18d;
	font_awesome_table["stack-overflow"] = 0xf16c;
	font_awesome_table["stackpath"] = 0xf842;
	font_awesome_table["staff-snake"] = 0xe579;
	font_awesome_table["stairs"] = 0xe289;
	font_awesome_table["stamp"] = 0xf5bf;
	font_awesome_table["stapler"] = 0xe5af;
	font_awesome_table["star"] = 0xf005;
	font_awesome_table["star-and-crescent"] = 0xf699;
	font_awesome_table["star-half"] = 0xf089;
	font_awesome_table["star-half-stroke"] = 0xf5c0;
	font_awesome_table["star-of-david"] = 0xf69a;
	font_awesome_table["star-of-life"] = 0xf621;
	font_awesome_table["staylinked"] = 0xf3f5;
	font_awesome_table["steam"] = 0xf1b6;
	font_awesome_table["steam-symbol"] = 0xf3f6;
	font_awesome_table["sterling-sign"] = 0xf154;
	font_awesome_table["stethoscope"] = 0xf0f1;
	font_awesome_table["sticker-mule"] = 0xf3f7;
	font_awesome_table["stop"] = 0xf04d;
	font_awesome_table["stopwatch"] = 0xf2f2;
	font_awesome_table["stopwatch-20"] = 0xe06f;
	font_awesome_table["store"] = 0xf54e;
	font_awesome_table["store-slash"] = 0xe071;
	font_awesome_table["strava"] = 0xf428;
	font_awesome_table["street-view"] = 0xf21d;
	font_awesome_table["strikethrough"] = 0xf0cc;
	font_awesome_table["stripe"] = 0xf429;
	font_awesome_table["stripe-s"] = 0xf42a;
	font_awesome_table["stroopwafel"] = 0xf551;
	font_awesome_table["stubber"] = 0xe5c7;
	font_awesome_table["studiovinari"] = 0xf3f8;
	font_awesome_table["stumbleupon"] = 0xf1a4;
	font_awesome_table["stumbleupon-circle"] = 0xf1a3;
	font_awesome_table["subscript"] = 0xf12c;
	font_awesome_table["suitcase"] = 0xf0f2;
	font_awesome_table["suitcase-medical"] = 0xf0fa;
	font_awesome_table["suitcase-rolling"] = 0xf5c1;
	font_awesome_table["sun"] = 0xf185;
	font_awesome_table["sun-plant-wilt"] = 0xe57a;
	font_awesome_table["superpowers"] = 0xf2dd;
	font_awesome_table["superscript"] = 0xf12b;
	font_awesome_table["supple"] = 0xf3f9;
	font_awesome_table["supportnow"] = 0xe833;
	font_awesome_table["suse"] = 0xf7d6;
	font_awesome_table["svelte"] = 0xe868;
	font_awesome_table["swatchbook"] = 0xf5c3;
	font_awesome_table["swift"] = 0xf8e1;
	font_awesome_table["symfony"] = 0xf83d;
	font_awesome_table["symfonycasts"] = 0xe8ab;
	font_awesome_table["synagogue"] = 0xf69b;
	font_awesome_table["syringe"] = 0xf48e;
	font_awesome_table["t"] = 0x54;
	font_awesome_table["table"] = 0xf0ce;
	font_awesome_table["table-cells"] = 0xf00a;
	font_awesome_table["table-cells-column-lock"] = 0xe678;
	font_awesome_table["table-cells-large"] = 0xf009;
	font_awesome_table["table-cells-row-lock"] = 0xe67a;
	font_awesome_table["table-cells-row-unlock"] = 0xe691;
	font_awesome_table["table-columns"] = 0xf0db;
	font_awesome_table["table-list"] = 0xf00b;
	font_awesome_table["table-tennis-paddle-ball"] = 0xf45d;
	font_awesome_table["tablet"] = 0xf3fb;
	font_awesome_table["tablet-button"] = 0xf10a;
	font_awesome_table["tablet-screen-button"] = 0xf3fa;
	font_awesome_table["tablets"] = 0xf490;
	font_awesome_table["tachograph-digital"] = 0xf566;
	font_awesome_table["tag"] = 0xf02b;
	font_awesome_table["tags"] = 0xf02c;
	font_awesome_table["tailwind-css"] = 0xe866;
	font_awesome_table["tape"] = 0xf4db;
	font_awesome_table["tarp"] = 0xe57b;
	font_awesome_table["tarp-droplet"] = 0xe57c;
	font_awesome_table["taurus"] = 0xe84f;
	font_awesome_table["taxi"] = 0xf1ba;
	font_awesome_table["teamspeak"] = 0xf4f9;
	font_awesome_table["teeth"] = 0xf62e;
	font_awesome_table["teeth-open"] = 0xf62f;
	font_awesome_table["telegram"] = 0xf2c6;
	font_awesome_table["temperature-arrow-down"] = 0xe03f;
	font_awesome_table["temperature-arrow-up"] = 0xe040;
	font_awesome_table["temperature-empty"] = 0xf2cb;
	font_awesome_table["temperature-full"] = 0xf2c7;
	font_awesome_table["temperature-half"] = 0xf2c9;
	font_awesome_table["temperature-high"] = 0xf769;
	font_awesome_table["temperature-low"] = 0xf76b;
	font_awesome_table["temperature-quarter"] = 0xf2ca;
	font_awesome_table["temperature-three-quarters"] = 0xf2c8;
	font_awesome_table["tencent-weibo"] = 0xf1d5;
	font_awesome_table["tenge-sign"] = 0xf7d7;
	font_awesome_table["tent"] = 0xe57d;
	font_awesome_table["tent-arrow-down-to-line"] = 0xe57e;
	font_awesome_table["tent-arrow-left-right"] = 0xe57f;
	font_awesome_table["tent-arrow-turn-left"] = 0xe580;
	font_awesome_table["tent-arrows-down"] = 0xe581;
	font_awesome_table["tents"] = 0xe582;
	font_awesome_table["terminal"] = 0xf120;
	font_awesome_table["tex"] = 0xe7ff;
	font_awesome_table["text-height"] = 0xf034;
	font_awesome_table["text-slash"] = 0xf87d;
	font_awesome_table["text-width"] = 0xf035;
	font_awesome_table["the-red-yeti"] = 0xf69d;
	font_awesome_table["themeco"] = 0xf5c6;
	font_awesome_table["themeisle"] = 0xf2b2;
	font_awesome_table["thermometer"] = 0xf491;
	font_awesome_table["think-peaks"] = 0xf731;
	font_awesome_table["threads"] = 0xe618;
	font_awesome_table["threema"] = 0xe85f;
	font_awesome_table["thumbs-down"] = 0xf165;
	font_awesome_table["thumbs-up"] = 0xf164;
	font_awesome_table["thumbtack"] = 0xf08d;
	font_awesome_table["thumbtack-slash"] = 0xe68f;
	font_awesome_table["ticket"] = 0xf145;
	font_awesome_table["ticket-simple"] = 0xf3ff;
	font_awesome_table["tidal"] = 0xe7dc;
	font_awesome_table["tiktok"] = 0xe07b;
	font_awesome_table["timeline"] = 0xe29c;
	font_awesome_table["toggle-off"] = 0xf204;
	font_awesome_table["toggle-on"] = 0xf205;
	font_awesome_table["toilet"] = 0xf7d8;
	font_awesome_table["toilet-paper"] = 0xf71e;
	font_awesome_table["toilet-paper-slash"] = 0xe072;
	font_awesome_table["toilet-portable"] = 0xe583;
	font_awesome_table["toilets-portable"] = 0xe584;
	font_awesome_table["toolbox"] = 0xf552;
	font_awesome_table["tooth"] = 0xf5c9;
	font_awesome_table["tor-browser"] = 0xe838;
	font_awesome_table["torii-gate"] = 0xf6a1;
	font_awesome_table["tornado"] = 0xf76f;
	font_awesome_table["tower-broadcast"] = 0xf519;
	font_awesome_table["tower-cell"] = 0xe585;
	font_awesome_table["tower-observation"] = 0xe586;
	font_awesome_table["tractor"] = 0xf722;
	font_awesome_table["trade-federation"] = 0xf513;
	font_awesome_table["trademark"] = 0xf25c;
	font_awesome_table["traffic-light"] = 0xf637;
	font_awesome_table["trailer"] = 0xe041;
	font_awesome_table["train"] = 0xf238;
	font_awesome_table["train-subway"] = 0xf239;
	font_awesome_table["train-tram"] = 0xe5b4;
	font_awesome_table["transgender"] = 0xf225;
	font_awesome_table["trash"] = 0xf1f8;
	font_awesome_table["trash-arrow-up"] = 0xf829;
	font_awesome_table["trash-can"] = 0xf2ed;
	font_awesome_table["trash-can-arrow-up"] = 0xf82a;
	font_awesome_table["tree"] = 0xf1bb;
	font_awesome_table["tree-city"] = 0xe587;
	font_awesome_table["trello"] = 0xf181;
	font_awesome_table["triangle-exclamation"] = 0xf071;
	font_awesome_table["trophy"] = 0xf091;
	font_awesome_table["trowel"] = 0xe589;
	font_awesome_table["trowel-bricks"] = 0xe58a;
	font_awesome_table["truck"] = 0xf0d1;
	font_awesome_table["truck-arrow-right"] = 0xe58b;
	font_awesome_table["truck-droplet"] = 0xe58c;
	font_awesome_table["truck-fast"] = 0xf48b;
	font_awesome_table["truck-field"] = 0xe58d;
	font_awesome_table["truck-field-un"] = 0xe58e;
	font_awesome_table["truck-front"] = 0xe2b7;
	font_awesome_table["truck-medical"] = 0xf0f9;
	font_awesome_table["truck-monster"] = 0xf63b;
	font_awesome_table["truck-moving"] = 0xf4df;
	font_awesome_table["truck-pickup"] = 0xf63c;
	font_awesome_table["truck-plane"] = 0xe58f;
	font_awesome_table["truck-ramp-box"] = 0xf4de;
	font_awesome_table["tty"] = 0xf1e4;
	font_awesome_table["tumblr"] = 0xf173;
	font_awesome_table["turkish-lira-sign"] = 0xe2bb;
	font_awesome_table["turn-down"] = 0xf3be;
	font_awesome_table["turn-up"] = 0xf3bf;
	font_awesome_table["tv"] = 0xf26c;
	font_awesome_table["twitch"] = 0xf1e8;
	font_awesome_table["twitter"] = 0xf099;
	font_awesome_table["typescript"] = 0xe840;
	font_awesome_table["typo3"] = 0xf42b;
	font_awesome_table["u"] = 0x55;
	font_awesome_table["uber"] = 0xf402;
	font_awesome_table["ubuntu"] = 0xf7df;
	font_awesome_table["uikit"] = 0xf403;
	font_awesome_table["ultralytics"] = 0xe86d;
	font_awesome_table["ultralytics-hub"] = 0xe86e;
	font_awesome_table["ultralytics-yolo"] = 0xe86f;
	font_awesome_table["umbraco"] = 0xf8e8;
	font_awesome_table["umbrella"] = 0xf0e9;
	font_awesome_table["umbrella-beach"] = 0xf5ca;
	font_awesome_table["uncharted"] = 0xe084;
	font_awesome_table["underline"] = 0xf0cd;
	font_awesome_table["uniregistry"] = 0xf404;
	font_awesome_table["unison"] = 0xe854;
	font_awesome_table["unity"] = 0xe049;
	font_awesome_table["universal-access"] = 0xf29a;
	font_awesome_table["unlock"] = 0xf09c;
	font_awesome_table["unlock-keyhole"] = 0xf13e;
	font_awesome_table["unreal-engine"] = 0xe85c;
	font_awesome_table["unsplash"] = 0xe07c;
	font_awesome_table["untappd"] = 0xf405;
	font_awesome_table["up-down"] = 0xf338;
	font_awesome_table["up-down-left-right"] = 0xf0b2;
	font_awesome_table["up-long"] = 0xf30c;
	font_awesome_table["up-right-and-down-left-from-center"] = 0xf424;
	font_awesome_table["up-right-from-square"] = 0xf35d;
	font_awesome_table["upload"] = 0xf093;
	font_awesome_table["ups"] = 0xf7e0;
	font_awesome_table["upwork"] = 0xe641;
	font_awesome_table["usb"] = 0xf287;
	font_awesome_table["user"] = 0xf007;
	font_awesome_table["user-astronaut"] = 0xf4fb;
	font_awesome_table["user-check"] = 0xf4fc;
	font_awesome_table["user-clock"] = 0xf4fd;
	font_awesome_table["user-doctor"] = 0xf0f0;
	font_awesome_table["user-gear"] = 0xf4fe;
	font_awesome_table["user-graduate"] = 0xf501;
	font_awesome_table["user-group"] = 0xf500;
	font_awesome_table["user-injured"] = 0xf728;
	font_awesome_table["user-lock"] = 0xf502;
	font_awesome_table["user-minus"] = 0xf503;
	font_awesome_table["user-ninja"] = 0xf504;
	font_awesome_table["user-nurse"] = 0xf82f;
	font_awesome_table["user-pen"] = 0xf4ff;
	font_awesome_table["user-plus"] = 0xf234;
	font_awesome_table["user-secret"] = 0xf21b;
	font_awesome_table["user-shield"] = 0xf505;
	font_awesome_table["user-slash"] = 0xf506;
	font_awesome_table["user-tag"] = 0xf507;
	font_awesome_table["user-tie"] = 0xf508;
	font_awesome_table["user-xmark"] = 0xf235;
	font_awesome_table["users"] = 0xf0c0;
	font_awesome_table["users-between-lines"] = 0xe591;
	font_awesome_table["users-gear"] = 0xf509;
	font_awesome_table["users-line"] = 0xe592;
	font_awesome_table["users-rays"] = 0xe593;
	font_awesome_table["users-rectangle"] = 0xe594;
	font_awesome_table["users-slash"] = 0xe073;
	font_awesome_table["users-viewfinder"] = 0xe595;
	font_awesome_table["usps"] = 0xf7e1;
	font_awesome_table["ussunnah"] = 0xf407;
	font_awesome_table["utensils"] = 0xf2e7;
	font_awesome_table["v"] = 0x56;
	font_awesome_table["vaadin"] = 0xf408;
	font_awesome_table["van-shuttle"] = 0xf5b6;
	font_awesome_table["vault"] = 0xe2c5;
	font_awesome_table["venmo"] = 0xe85a;
	font_awesome_table["venmo-v"] = 0xe85b;
	font_awesome_table["venus"] = 0xf221;
	font_awesome_table["venus-double"] = 0xf226;
	font_awesome_table["venus-mars"] = 0xf228;
	font_awesome_table["vest"] = 0xe085;
	font_awesome_table["vest-patches"] = 0xe086;
	font_awesome_table["viacoin"] = 0xf237;
	font_awesome_table["viadeo"] = 0xf2a9;
	font_awesome_table["vial"] = 0xf492;
	font_awesome_table["vial-circle-check"] = 0xe596;
	font_awesome_table["vial-virus"] = 0xe597;
	font_awesome_table["vials"] = 0xf493;
	font_awesome_table["viber"] = 0xf409;
	font_awesome_table["video"] = 0xf03d;
	font_awesome_table["video-slash"] = 0xf4e2;
	font_awesome_table["vihara"] = 0xf6a7;
	font_awesome_table["vim"] = 0xe88a;
	font_awesome_table["vimeo"] = 0xf40a;
	font_awesome_table["vimeo-v"] = 0xf27d;
	font_awesome_table["vine"] = 0xf1ca;
	font_awesome_table["virgo"] = 0xe850;
	font_awesome_table["virus"] = 0xe074;
	font_awesome_table["virus-covid"] = 0xe4a8;
	font_awesome_table["virus-covid-slash"] = 0xe4a9;
	font_awesome_table["virus-slash"] = 0xe075;
	font_awesome_table["viruses"] = 0xe076;
	font_awesome_table["vk"] = 0xf189;
	font_awesome_table["vnv"] = 0xf40b;
	font_awesome_table["voicemail"] = 0xf897;
	font_awesome_table["volcano"] = 0xf770;
	font_awesome_table["volleyball"] = 0xf45f;
	font_awesome_table["volume"] = 0xf6a8;
	font_awesome_table["volume-high"] = 0xf028;
	font_awesome_table["volume-low"] = 0xf027;
	font_awesome_table["volume-off"] = 0xf026;
	font_awesome_table["volume-xmark"] = 0xf6a9;
	font_awesome_table["vr-cardboard"] = 0xf729;
	font_awesome_table["vsco"] = 0xe7dd;
	font_awesome_table["vuejs"] = 0xf41f;
	font_awesome_table["w"] = 0x57;
	font_awesome_table["w3c"] = 0xe7de;
	font_awesome_table["walkie-talkie"] = 0xf8ef;
	font_awesome_table["wallet"] = 0xf555;
	font_awesome_table["wand-magic"] = 0xf0d0;
	font_awesome_table["wand-magic-sparkles"] = 0xe2ca;
	font_awesome_table["wand-sparkles"] = 0xf72b;
	font_awesome_table["warehouse"] = 0xf494;
	font_awesome_table["watchman-monitoring"] = 0xe087;
	font_awesome_table["water"] = 0xf773;
	font_awesome_table["water-ladder"] = 0xf5c5;
	font_awesome_table["wave-square"] = 0xf83e;
	font_awesome_table["waze"] = 0xf83f;
	font_awesome_table["web-awesome"] = 0xe682;
	font_awesome_table["webflow"] = 0xe65c;
	font_awesome_table["weebly"] = 0xf5cc;
	font_awesome_table["weibo"] = 0xf18a;
	font_awesome_table["weight-hanging"] = 0xf5cd;
	font_awesome_table["weight-scale"] = 0xf496;
	font_awesome_table["weixin"] = 0xf1d7;
	font_awesome_table["whatsapp"] = 0xf232;
	font_awesome_table["wheat-awn"] = 0xe2cd;
	font_awesome_table["wheat-awn-circle-exclamation"] = 0xe598;
	font_awesome_table["wheelchair"] = 0xf193;
	font_awesome_table["wheelchair-move"] = 0xe2ce;
	font_awesome_table["whiskey-glass"] = 0xf7a0;
	font_awesome_table["whmcs"] = 0xf40d;
	font_awesome_table["wifi"] = 0xf1eb;
	font_awesome_table["wikipedia-w"] = 0xf266;
	font_awesome_table["wind"] = 0xf72e;
	font_awesome_table["window-maximize"] = 0xf2d0;
	font_awesome_table["window-minimize"] = 0xf2d1;
	font_awesome_table["window-restore"] = 0xf2d2;
	font_awesome_table["windows"] = 0xf17a;
	font_awesome_table["wine-bottle"] = 0xf72f;
	font_awesome_table["wine-glass"] = 0xf4e3;
	font_awesome_table["wine-glass-empty"] = 0xf5ce;
	font_awesome_table["wirsindhandwerk"] = 0xe2d0;
	font_awesome_table["wix"] = 0xf5cf;
	font_awesome_table["wizards-of-the-coast"] = 0xf730;
	font_awesome_table["wodu"] = 0xe088;
	font_awesome_table["wolf-pack-battalion"] = 0xf514;
	font_awesome_table["won-sign"] = 0xf159;
	font_awesome_table["wordpress"] = 0xf19a;
	font_awesome_table["wordpress-simple"] = 0xf411;
	font_awesome_table["worm"] = 0xe599;
	font_awesome_table["wpbeginner"] = 0xf297;
	font_awesome_table["wpexplorer"] = 0xf2de;
	font_awesome_table["wpforms"] = 0xf298;
	font_awesome_table["wpressr"] = 0xf3e4;
	font_awesome_table["wrench"] = 0xf0ad;
	font_awesome_table["x"] = 0x58;
	font_awesome_table["x-ray"] = 0xf497;
	font_awesome_table["x-twitter"] = 0xe61b;
	font_awesome_table["xbox"] = 0xf412;
	font_awesome_table["xing"] = 0xf168;
	font_awesome_table["xmark"] = 0xf00d;
	font_awesome_table["xmarks-lines"] = 0xe59a;
	font_awesome_table["xmpp"] = 0xe864;
	font_awesome_table["y"] = 0x59;
	font_awesome_table["y-combinator"] = 0xf23b;
	font_awesome_table["yahoo"] = 0xf19e;
	font_awesome_table["yammer"] = 0xf840;
	font_awesome_table["yandex"] = 0xf413;
	font_awesome_table["yandex-international"] = 0xf414;
	font_awesome_table["yarn"] = 0xf7e3;
	font_awesome_table["yelp"] = 0xf1e9;
	font_awesome_table["yen-sign"] = 0xf157;
	font_awesome_table["yin-yang"] = 0xf6ad;
	font_awesome_table["yoast"] = 0xf2b1;
	font_awesome_table["youtube"] = 0xf167;
	font_awesome_table["z"] = 0x5a;
	font_awesome_table["zhihu"] = 0xf63f;
	font_awesome_table["zoom"] = 0xe87b;
	font_awesome_table["zulip"] = 0xe853;
    }
}

wchar_t icon_wchar(const char* name) {
    auto it = font_awesome_table.find(name);
    if(it == font_awesome_table.end()) {
        return '\0';
    }
    return it->second;
}

void init_icon_table() {
    init_font_awesome_table();
}

namespace GEO {
    std::string icon_UTF8(const char* name) {
        wchar_t result[2];
        result[0] = icon_wchar(name);
        result[1] = '\0';
        return String::wchar_to_UTF8(result);
    }
}
